#!/usr/bin/env python2
"""RNAalignment - a module to work with RNA sequence alignments.

To see a full demo what you can do with this util, please take a look at the jupiter notebook (https://github.com/mmagnus/rna-pdb-tools/blob/master/rna_tools/tools/rna_alignment/rna_alignment.ipynb)

Load an alignment in the Stockholm::

    alignment = ra.RNAalignment('test_data/RF00167.stockholm.sto')

 or fasta format::

     import rna_alignment as ra
     alignment = ra.fasta2stokholm(alignment.fasta)
     alignment = ra.RNAalignment

Parameters of the aligmnent::

     print(alignment.describe())

Consensus SS::

    print(alignment.ss_cons_with_pk)

Get sequnce/s from teh aligment::

    >>> seq = a.io[0]

"""

from Bio import AlignIO
from Bio.Seq import Seq
from Bio.SeqRecord import SeqRecord
from Bio.Phylo.TreeConstruction import DistanceCalculator
from rna_tools import SecondaryStructure
from rna_tools.rna_tools_config import RCHIE_PATH
from collections import OrderedDict

import Levenshtein
import tempfile
import subprocess
import os
import shutil
import re
import gzip
import copy


class RNAalignmentError(Exception):
    pass


class RChieError(Exception):
    pass


class RFAMFetchError(Exception):
    pass


class RChie:
    """RChie - plotting arc diagrams of RNA secondary structures.

    .. image:: ../pngs/rchie.png

    http://www.e-rna.org/r-chie/

    The offline version of R-chie, which requires first installing R4RNA is available here, or clone our git repository here
    How to install it:

    - Ensure R is installed already, or download it freely from http://www.r-project.org/
    - Download the R4RNA (https://github.com/jujubix/r-chie), open R and install the package::

            install.packages("<path_to_file>/R4RNA", repos = NULL, type="source")
            # Install the optparse and RColorBrewer
            install.packages('optparse')
            install.packages('RColorBrewer')

    - Go to rna_tools/rna_tools_config_local.py and set RCHIE_PATH to the folder with RChie, e.g. ``"/home/magnus/work/opt/r-chie/"``.

    To test if Rchie works on your machine (from rna_align folder)::

      <path to your rchie>/rchie.R --msafile test_data/rchie_test_files/fasta.txt test_data/rchie_test_files/helix.txt

    you should have rchie.png file in the folder.

    More at http://www.e-rna.org/r-chie/download.cgi

    Cite: Daniel Lai, Jeff R. Proctor, Jing Yun A. Zhu, and Irmtraud M. Meyer (2012) R-chie: a web server and R package for visualizing RNA secondary structures. Nucleic Acids Research, first published online March 19, 2012. doi:10.1093/nar/gks241
    """

    def __init__(self):
        pass

    def plot_cov(self, seqs, ss_cons, plot_fn='rchie.png', verbose=False):
        """Plot an RChie plot_conv.

        :param seqs: seqs in the fasta format
        :param ss_cons: a string of secondary structure consensus, use only ``().``. Works with pseuoknots.
        """
        fasta_alignment = tempfile.NamedTemporaryFile(delete=False)
        with open(fasta_alignment.name, 'w') as f:
            f.write(seqs)

        plot = tempfile.NamedTemporaryFile(delete=False)
        plot.name += '.png'
        ss = tempfile.NamedTemporaryFile(delete=False)
        with open(ss.name, 'w') as f:
            f.write(ss_cons)
        if not RCHIE_PATH:
            raise RChieError('RChie path not set up!')
        cmd = RCHIE_PATH + \
            "rchie.R --msafile='%s' --format1 vienna '%s' --output '%s'" % (
                fasta_alignment.name, ss.name, plot.name)
        if verbose:
            print(cmd)
        o = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out = o.stdout.read().strip()
        err = o.stderr.read().strip()
        # *****PROCESSING MSA FILE*****
        # Error in readFasta(opt$msafile, filter = TRUE) : no FASTA sequences found
        # Error: ERROR: Invalid FASTA file
        # Execution halted
        if "error" in str(err).lower():
            raise Exception('\n'.join([cmd, err]))
        if verbose:
            print('\n'.join([cmd, err]))
        self.plotfn = plot.name
        if verbose:
            print(self.plotfn)
        if plot_fn:
            shutil.move(plot.name, plot_fn)
            print('Rchie: plot saved to %s' % plot_fn)
        from IPython.display import Image
        return Image(filename=plot_fn)

    def show(self):
        from IPython.display import Image
        return Image(filename=self.plotfn)

    def write(self, outfn):
        shutil.copyfile(self.plotfn, outfn)
        print('Write to %s' % outfn)


class RNASeq(object):
    """RNASeq.

    Args:

       id (str)    : id of a sequence
       seq (str)   : seq, it be uppercased.
       ss (str)    : secondary structure, default None

    Attributes:

       seq_no_gaps(str) : seq.replace('-', '')
       ss_no_gaps(str)  : ss.replace('-', '')

    """

    def __init__(self, id, seq, ss=None):
        self.id = id
        self.seq = seq.upper()

        # self.ss_raw = ss  # this will not be changed after remove_gaps.
        # so maybe don't use ss_raw at call
        self.ss = ss
        self.ss = self.get_ss_std()

        self.seq_no_gaps = seq.replace('-', '')
        self.ss_no_gaps = ss.replace('-', '')

    #@property
    def get_ss_std(self):
        nss = ''
        for s in self.ss:
            nss += get_rfam_ss_notat_to_dot_bracket_notat(s)
        return nss

    def __repr__(self):
        return self.id

    def __len__(self):
        return len(self.seq)

    def __getitem__(self, i):
        if self.ss:
            return RNASeq(self.id + '_slice', self.seq[i], self.ss[i])
        else:
            return RNASeq(self.id + '_slice', self.seq[i])

    def remove_columns(self, to_remove):
        """indexing from 0"""
        nseq = ''
        for i, s in enumerate(self.seq):
            if i not in to_remove:
                nseq += s

        nss = ''
        if self.ss:
            for i, s in enumerate(self.ss):
                if i not in to_remove:
                    nss += s
        self.seq = nseq
        self.ss = nss

    def draw_ss(self, title='', verbose=False, resolution=1.5):
        """Draw secondary structure of RNA with VARNA.

        VARNA: Visualization Applet for RNA
        A Java lightweight component and applet for drawing the RNA secondary structure

        .. image :: ../pngs/varna.png

        Cite: VARNA: Interactive drawing and editing of the RNA secondary structure Kevin Darty, Alain Denise and Yann Ponty Bioinformatics, pp. 1974-197,, Vol. 25, no. 15, 2009

        http://varna.lri.fr/"""
        drawfn = tempfile.NamedTemporaryFile(delete=False).name + '.png'
        SecondaryStructure.draw_ss(title, self.seq, self.ss, drawfn, resolution, verbose=verbose)
        from IPython.display import Image
        return Image(filename=drawfn)

    def remove_gaps(self, check_bps=True, only_canonical=True, allow_gu=True):
        """Remove gaps from seq and secondary structure of the seq.

        Args:

            check_bps (bool)      : fix mistakes as
            only_canonical (bool) : keep in ss only pairs GC, AU
            allow_gu (bool)       : keep in ss also GU pair

        .. image :: ../pngs/ss_misgap.png

        A residue "paired" with a gap.

        .. image :: ../pngs/ss_misgap_wrong.png

        .. when check_bps (by default), then when removing gaps, the functions check is the gap is
        paired with any residues (in the blue circle). If yes, then this residues is unpair (in this case ``)`` -> ``.``).

        .. image :: ../pngs/ss_misgap_ok.png

        if ``only_canonical`` (by default) is True then only GC, AU can be paired.


        .. image :: ../pngs/ss_only_canonical.png


        If ``allow_gu`` is False (be default is True) then GU pair is also possible.

        .. image :: ../pngs/ss_no_gu.png

        If you provide seq and secondary structure such as::

             GgCcGGggG.GcggG.cc.u.aAUACAAuACCC.GaAA.GGGGAAUAaggCc.gGCc.gu......CU.......uugugcgGUuUUcaAgCccCCgGcCaCCcuuuu
             (((((((((....((.((............(((......)))......))))..(((.(.....................)))).......)))))))))........

        gaps will be remove as well.
        """
        GAPS = ['-', '.']

        nseq = ''
        nss = ''
        for (nt, nt_ss) in zip(self.seq, self.ss):
            if nt in GAPS and nt_ss in GAPS:
                pass
            else:
                nseq += nt
                nss += nt_ss
        self.seq = nseq
        self.ss = nss

        nss = list()
        bps = self.ss_to_bps()

        if check_bps:
            nss = list(self.ss)
            for bp in bps:
                nt_left = self.seq[bp[0]]
                nt_right = self.seq[bp[1]]
                if nt_left == '-' or nt_right == '-':
                    nss[bp[0]] = '.'
                    nss[bp[1]] = '.'
            self.ss = ''.join(nss)

        if only_canonical:
            nseq = list(self.seq)
            nss = list(self.ss)
            for bp in bps:
                nt_left = nseq[bp[0]]
                nt_right = nseq[bp[1]]
                if (nt_left == 'A' and nt_right == 'U') or (nt_left == 'U' and nt_right == 'A'):
                    pass
                elif (nt_left == 'G' and nt_right == 'C') or (nt_left == 'C' and nt_right == 'G'):
                    pass
                elif (nt_left == 'G' and nt_right == 'U') or (nt_left == 'U' and nt_right == 'G'):
                    if allow_gu:
                        pass
                    else:
                        nss[bp[0]] = '.'
                        nss[bp[1]] = '.'
                else:
                    nss[bp[0]] = '.'
                    nss[bp[1]] = '.'
            self.ss = ''.join(nss)

        # two?????? what is "two"?
        nss = []
        nseq = ''
        for i, (c, s) in enumerate(zip(self.seq, self.ss)):
            if c != '-':
                nseq += c
                nss.append(s)

        self.seq = nseq
        self.ss = ''.join(nss)

    def ss_to_bps(self):
        """Convert secondary structure into a list of basepairs.

        Returns:

            bps (list): a list of base pairs, e.g. [[0, 80], [1, 79], [2, 78], [4, 77], [6, 75], [7, 74], ...]

        """
        j = []
        bps = []
        pair_types = ['()', '[]', '<>', '{}']
        for pair_type in pair_types:
            for i, s in enumerate(self.ss):
                if s == pair_type[0]:
                    j.append(i)
                if s == pair_type[1]:
                    bps.append([j.pop(), i])
            if len(j):
                # if something left, this is a problem (!!!)
                raise Exception('Mis-paired secondary structure')
        bps.sort()
        return bps

    def get_conserved(self, consensus, start=0, to_pymol=True, offset=0):
        """Start
        UCGGGGUGCCCUUCUGCGUG--------------------------------------------------AAGGC-UGAGAAAUACCCGU-------------------------------------------------AUCACCUG-AUCUGGAU-AAUGC
        XXXXXXXXXXXXGXGXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX----------------------------XXXXX-XCUGAGAXXXXXXXXXXXXXXXXXXXXXX----------------------------------XXXXXXXX-XXXXXXXX-ACXUG
        """
        c = start + offset
        index = []
        print(self.seq)
        print(consensus)
        for nt_seq, nt_consensus in zip(self.seq, consensus):
            if nt_consensus in ['-', 'X']:
                pass
            else:
                index.append(c)
            if nt_seq != '-':
                c += 1
        if to_pymol:
            return("color red, " + str(index).replace(', ', '+').replace('[','').replace(']',''))
        else:
            return index

    def get_distance_to(self, nseq):
        """Get distance of self.seq to nseq."""
        return round(Levenshtein.ratio(self.seq, nseq), 2)

class RNAalignment(object):
    """RNA alignment - adapter class around BioPython to do RNA alignment stuff

    Usage (for more see IPython notebook https://github.com/mmagnus/rna-tools/blob/master/rna_tools/tools/rna_alignment/rna_alignment.ipynb)

    >>> a = RNAalignment('test_data/RF00167.stockholm.sto')
    >>> print(a.tail())
    >>> print(a.ss_cons)

    Args:

      fn (str): Filename
      io (Bio.AlignIO): AlignIO.read(fn, "stockholm")
      lines (list): List of all lines of fn
      seqs (list): List of all sequences as class:`RNASeq` objects
      rf (str): ReFerence annotation, the consensus RNA sequence

    Read more:

    - http://biopython.org/DIST/docs/api/Bio.AlignIO.StockholmIO-module.html

    and on the format itself

    - https://en.wikipedia.org/wiki/Stockholm_format
    - http://sonnhammer.sbc.su.se/Stockholm.html

    .. warning:: fetch requires urllib3
    """

    def __init__(self, fn='', fetch=''):
        if fetch:
            import urllib3
            http = urllib3.PoolManager()
            response = http.request('GET', 'http://rfam.xfam.org/family/' +
                                    fetch + '/alignment/stockholm?gzip=1&download=1')
            if not response.status == 200:
                raise RFAMFetchError(
                    "The alignment could not be downloaded. Please check the RFAM id that you requested! (don't put .stk etc in the id)")
            with open(fetch + '.stk.gz', 'wb') as f:
                f.write(response.data)
            with gzip.open(fetch + '.stk.gz', 'rb') as f:
                file_content = f.read()
            with open(fetch + '.stk', 'wb') as f:
                f.write(file_content)
            fn = fetch + '.stk'

        self.fn = fn
        self.lines = open(fn).read().split('\n')
        self.io = AlignIO.read(fn, "stockholm")
        self.ss_cons = self.get_ss_cons()
        self.ss_cons_pk = self.get_ss_cons_pk()
        self.copy_ss_cons_to_all()
        self._ss_cons_std = self.ss_cons
        self.rf = self.get_gc_rf()
        self.shift = self.get_shift_seq_in_align()

        # get all lines not # nor //
        # fix for blocked alignment
        seq_lines = [l for l in self.lines if (
            not l.startswith('#')) and (not l.startswith('//')) and (l)]
        seqs_dict = OrderedDict()
        for seq in seq_lines:
            seq_id, seq_seq = seq.split()
            if seq_id not in seqs_dict:
                seqs_dict[seq_id] = seq_seq
            else:
                seqs_dict[seq_id] += seq_seq
        self.seqs = []
        for seq in seqs_dict:
            self.seqs.append(RNASeq(seq, seqs_dict[seq], ss=self.ss_cons_with_pk_std))

        # this is sick!
        # I create a Cols object to be able to slice alignments
        class Cols:
            def __init__(self, alignment):
                self.alignment = alignment

            def __getitem__(self, i):
                """Return new alignment"""
                if type(i) is list:
                    pass
                else:
                    # collect "new" sequences
                    n_seqs = []
                    for s in self.alignment:
                        new_seq = RNASeq(s.id, s.seq[i], s.ss[i])
                        n_seqs.append(new_seq)

                    # this is not very smart :(
                    # save new seqs to a file
                    # and load it as RNAalignment
                    tf = tempfile.NamedTemporaryFile(delete=False)
                    tf.name += '.stk'
                    with open(tf.name, 'w') as f:
                        f.write('# STOCKHOLM 1.0\n')
                        for s in n_seqs:
                            f.write(' '.join([s.id, s.seq, '\n']))
                        # add ss_cons & //
                        f.write('#=GC SS_cons ' + self.alignment.ss_cons[i] + '\n')
                        if self.alignment.ss_cons_pk:
                            f.write('#=GC SS_cons_pk' + self.alignment.ss_cons_pk[i] + '\n')
                        f.write('#=GC RF ' + self.alignment.rf[i] + '\n')
                        f.write('//\n')
                    return RNAalignment(tf.name)
        self.cols = Cols(self)
        # ^^^^ sick ^^^^^^^^^^^

    def reload_alignment(self):
        tmpfn = tempfile.NamedTemporaryFile(delete=False).name
        self.write(tmpfn)
        self.io = AlignIO.read(tmpfn, "stockholm")

    def __len__(self):
        """Return length of all sequenes."""
        return len(self.seqs)

    def __getitem__(self, i):
        if type(i) is str:
            for s in self:
                if s.id == i:
                    return s
        elif type(i) is list:
            seqs = []
            for j in i:
                seqs.append(self.seqs[j])
            return seqs
        else:
            return self.seqs[i]

    @property
    def ss_cons_std(self):
        return get_rfam_ss_notat_to_dot_bracket_notat(self.ss_cons)

    @ss_cons_std.setter
    def ss_cons_std(self, ss):
        self._ss_cons_std = ss
        print(self._ss_cons_std)

    def subset(self, ids, verbose=False):
        """Get subset for ids::

            # STOCKHOLM 1.0
            #=GF WK Tetrahydrofolate_riboswitch
            ..
            AAQK01002704.1/947-1059              -U-GC-AAAAUAGGUUUCCAUGC..
            #=GC SS_cons                         .(.((.((----((((((((((...
            #=GC RF                              .g.gc.aGAGUAGggugccgugc..
            //

        """
        nalign = ''
        for l in self.lines:
            if l.startswith('//'):
                nalign += l + '\n'
            if l.startswith('#'):
                nalign += l + '\n'
            else:
                for i in ids:
                    if l.startswith(i):
                        nalign += l + '\n'

        tf = tempfile.NamedTemporaryFile(delete=False)
        tf.name += '.stk'
        print('Saved to ', tf.name)
        if verbose:
            print(nalign)
        f = open(tf.name, 'w')
        f.write(nalign)
        #f.write(self.rf + '\n')
        f.close()
        return RNAalignment(tf.name)

    def __add__(self, rna_seq):
        self.seqs.append(rna_seq)
        self.reload_alignment()

    def write(self, fn, verbose=False):
        """Write the alignment to a file"""
        if verbose:
            print('Save to ', fn)
        with open(fn, 'w') as f:
            f.write('# STOCKHOLM 1.0\n')
            shift = max([len(x) for x in [s.id for s in self.seqs] + ['#=GC=SS_cons']])
            for s in self:
                f.write(s.id.ljust(shift + 2, ' ') + s.seq + '\n')
            f.write('#=GC SS_cons'.ljust(shift + 2, ' ') + self.ss_cons + '\n')
            f.write('//')

    def copy_ss_cons_to_all(self, verbose=False):
        for s in self.io:
            if verbose:
                self.ss_cons
                self.io[0].seq
            try:
                s.letter_annotations['secondary_structure'] = self.ss_cons
            except TypeError:
                raise Exception(
                    'Please check if all your sequences and ss lines are of the same length!')
            s.ss = self.ss_cons
            s.ss_clean = self.get_clean_ss(s.ss)
            s.seq_nogaps = str(s.seq).replace('-', '')
            s.ss_nogaps = self.get_ss_remove_gaps(s.seq, s.ss_clean)

    def copy_ss_cons_to_all_editing_sequence(self, seq_id, before, after):
        """Change a sequence's sec structure.

        :param seq_id: string, sequence id to change, eg: ``AE009948.1/1094322-1094400``
        :param before: string, character to change from, eg: ``,``
        :param after: string, character to change to, eg: ``.``

        .. warning:: before and after has to be one character long
        """
        for s in self.io:
            if s.id == seq_id:
                s.letter_annotations['secondary_structure'] = self.ss_cons.replace(before, after)
            else:
                s.letter_annotations['secondary_structure'] = self.ss_cons

    def get_ss_remove_gaps(self, seq, ss):
        """
        :param seq: string, sequence
        :param ss: string, ss

        UAU-AACAUAUAAUUUUGACAAUAUGG-GUCAUAA-GUUUCUACCGGAAUACC--GUAAAUAUUCU---GACUAUG-UAUA-
        (((.(.((((,,,(((((((_______.))))))).,,,,,,,,(((((((__.._____))))))...),,)))).)))).
        """
        nss = ''
        for i, j in zip(seq, ss):
            if i != '-':
                nss += j
        return nss

    def plot(self, plot_fn='rchie.png', verbose=False):
        return RChie().plot_cov(self.io.format("fasta"), self.ss_cons_std, plot_fn, verbose)

    def get_ss_cons_pk(self):
        """
        :return: SS_cons_pk line or None if there is now SS_cons_pk:"""
        ss_cons_pk = ''
        for l in self.lines:
            if l.startswith('#=GC SS_cons_pk'):
                ss_cons_pk += l.replace('#=GC SS_cons_pk', '').strip()
        return ss_cons_pk

    def get_ss_cons(self):
        """
        :return: SS_cons_pk line or None if there is now SS_cons_pk.
        """
        ss_cons = ''
        for l in self.lines:
            if '#=GC SS_cons' in l and '#=GC SS_cons_pk' not in l:
                ss_cons += l.replace('#=GC SS_cons', '').strip()
        return ss_cons

    @property
    def ss_cons_with_pk(self):
        """go over ss_cons and overwrite bp is there is pk (ss_cons_pk)

        ss_cons:         (((.(.((((,,,(((((((_______.))))))).,,,,,,,,(((((((__.._____))))))...),,)))).)))).
        ss_cons_pk:      .......................[[...............................]]........................
        ss_cons_with_pk: (((.(.((((,,,(((((((___[[__.))))))).,,,,,,,,(((((((__.._]]__))))))...),,)))).)))).

        "return ss_cons_with_pk: string, e.g. (((.(.((((,,,(((((((___[[__.))))"""
        if self.ss_cons_pk:
            ss_cons_with_pk = ''
            for i, (s, p) in enumerate(zip(self.ss_cons, self.ss_cons_pk)):
                if p != '.':
                    ss_cons_with_pk += p
                else:
                    ss_cons_with_pk += s
            return ss_cons_with_pk
        else:
            return self.ss_cons

    @property
    def ss_cons_with_pk_std(self):
        if self.ss_cons_pk:
            ss_cons_with_pk_std = ''
            for i, (s, p) in enumerate(zip(get_rfam_ss_notat_to_dot_bracket_notat(self.ss_cons), self.ss_cons_pk)):
                if p != '.':
                    ss_cons_with_pk_std += p
                else:
                    ss_cons_with_pk_std += s
            return ss_cons_with_pk_std
        else:
            return self.ss_cons

    def get_gc_rf(self):
        """Return (str) ``#=GC RF`` or '' if this line is not in the alignment.
        """
        for l in self.lines:
            if l.startswith('#=GC RF'):
                return l.replace('#=GC RF', '').replace('_cons', '').strip()
        else:
            return ''
            # raise RNAalignmentError('There is on #=GC RF in the alignment!')

    def get_shift_seq_in_align(self):
        """RF_cons vs '#=GC RF' ???"""
        for l in self.lines:
            if l.startswith('#=GC RF'):
                # #=GC RF                        .g.gc.a
                l = l.replace('#=GC RF', '')
                c = 7  # 12 # len of '#=GC RF'
                #                         .g.gc.a
                for i in l:
                    if i == ' ':
                        c += 1
                self.shift = c
                return c

    def map_seq_on_seq(self, seq_id, seq_id_target, resis, v=True):
        """
        :param seq_id: seq_id, 'AAML04000013.1/228868-228953'
        :param seq_id_target: seq_id of target, 'CP000721.1/2204691-2204778'
        :param resis: list resis, [5,6]

        map::

            [4, 5, 6]
            UAU-A
            UAU-AA
            UAU-AAC
            [5, 6, 7]
            CAC-U
            CAC-U-
            CAC-U-U
            [4, None, 5]

        """
        # get number for first seq
        print(resis)
        nresis = []
        for s in self.io:
            if s.id.strip() == seq_id.strip():
                for i in resis:
                    print(s.seq[:i + 1])  # UAU-A
                    nresis.append(i + s.seq[:i].count('-'))
        print(nresis)

        print(self.map_seq_on_align(seq_id_target, nresis))
        return

        resis_target = []
        for s in self.io:
            if s.id.strip() == seq_id_target.strip():
                for i in nresis:
                    if v:
                        print(s.seq[:i])
                    if s.seq[i - 1] == '-':
                        resis_target.append(None)
                    else:
                        resis_target.append(i - s.seq[:i].count('-'))
        return resis_target

    def map_seq_on_align(self, seq_id, resis, v=True):
        """
        :param seqid: seq_id, 'CP000721.1/2204691-2204775'
        :param resis: list resis, [5,6]

        maps::

            [5, 6, 8]
            CAC-U
            CAC-U-
            CAC-U-UA
            [4, None, 6]

        """
        if v:
            print(resis)
        nresis = []
        for s in self.io:
            if s.id.strip() == seq_id.strip():
                for i in resis:
                    if v:
                        print(s.seq[:i])
                    if s.seq[i - 1] == '-':
                        nresis.append(None)
                    else:
                        nresis.append(i - s.seq[:i].count('-'))
        return nresis

    def head(self):
        return '\n'.join(self.lines[:5])

    def tail(self):
        return '\n'.join(self.lines[-5:])

    def describe(self):
        """Describe the alignment.

           > print(a.describe())
           SingleLetterAlphabet() alignment with 13 rows and 82 columns

        """
        return str(self.io).split('\n')[0]

    def remove_empty_columns(self, verbose=False):
        """Remove empty columns in place.

        Example::

            >>> a = RNAalignment("test_data/zmp.stk")
            >>> print(a)
            SingleLetterAlphabet() alignment with 6 rows and 319 columns
            ---ACCUUGCGCGACUGGCGAAUCC-------------------...AAU CP001644.1/756294-756165
            --GCUCUCGCGCGACUGGCGACUUUG------------------...GAA CU234118.1/352539-352459
            UGAGUUUUCUGCGACUGACGGAUUAU------------------...CUG BAAV01000055.1/2897-2982
            GCCCGUUCGCGUGACUGGCGCUAGU-------------------...CGA CP000927.1/5164264-5164343
            -----GGGUCGUGACUGGCGAACA--------------------...--- zmp
            UCACCCCUGCGUGACUGGCGAUA---------------------...GUU AP009385.1/718103-718202
            >>> a.remove_empty_columns()
            >>> print(a)
            SingleLetterAlphabet() alignment with 6 rows and 138 columns
            ---ACCUUGCGCGACUGGCGAAUCC-UGAAGCUGCUUUG-AGCG...AAU CP001644.1/756294-756165
            --GCUCUCGCGCGACUGGCGACUUUG------------------...GAA CU234118.1/352539-352459
            UGAGUUUUCUGCGACUGACGGAUUAU------------------...CUG BAAV01000055.1/2897-2982
            GCCCGUUCGCGUGACUGGCGCUAGU-------------------...CGA CP000927.1/5164264-5164343
            -----GGGUCGUGACUGGCGAACA--------G-----------...--- zmp
            UCACCCCUGCGUGACUGGCGAUA--------GAACCCUCGGGUU...GUU AP009385.1/718103-718202

        go over all seq
        modifes self.nss_cons"""
        cols_to_rm = []

        # get only seqs
        for i in range(len(self[0])):
            gap = True
            for seq in self:
                if seq.seq[i] != '-':
                    gap = False
            if gap:
                cols_to_rm.append(i)

        # remove from sequences
        for s in self:
            s.remove_columns(cols_to_rm)

        # update io # hack #
        tmpfn = tempfile.NamedTemporaryFile(delete=False).name
        self.write(tmpfn, verbose=False)
        self.io = AlignIO.read(tmpfn, "stockholm")

        # nss_cons update
        nss_cons = ''
        for i, s in enumerate(self.ss_cons):
            if i not in cols_to_rm:
                nss_cons += s
        self.ss_cons = nss_cons

    def format_annotation(self, t):
        return self.shift * ' ' + t

    def find_core(self, ids=None):
        """Find common core for ids.

        .. image:: ../pngs/find_core.png
        Fig. By core, we understand columns that have all homologous residues. The core is here marked by `x`.

        :param id: list, ids of seq in the alignment to use
        """
        if not ids:
            ids = []
            for s in self.io:
                ids.append(s.id)

        xx = list(range(0, len(self.io[0])))
        for i in range(0, len(self.io[0])):  # if . don't use it
            for s in self.io:
                # print s.id
                if s.id in ids:
                    if s.seq[i] == '-':
                        xx[i] = '-'
                        break
                    else:
                        xx[i] = 'x'

        return ''.join(xx)
        shift = self.get_shift_seq_in_align()

        fnlist = open(self.fn).read().strip().split('\n')
        fnlist.insert(-2, 'x' + ' ' * (shift - 1) + ''.join(xx))
        # print fnlist
        for l in fnlist:
            print(l)

    def find_seq(self, seq, verbose=False):
        """Find seq (also subsequences) and reverse in the alignment.

        Args:

          seq (str): seq is upper()
          verbose (bool): be verbose

        ::

            seq = "ggaucgcugaacccgaaaggggcgggggacccagaaauggggcgaaucucuuccgaaaggaagaguaggguuacuccuucgacccgagcccgucagcuaaccucgcaagcguccgaaggagaauc"
            hit = a.find_seq(seq, verbose=False)
            ggaucgcugaacccgaaaggggcgggggacccagaaauggggcgaaucucuuccgaaaggaagaguaggguuacuccuucgacccgagcccgucagcuaaccucgcaagcguccgaaggagaauc
            Match: AL939120.1/174742-174619
            ID: AL939120.1/174742-174619
            Name: AL939120.1
            Description: AL939120.1/174742-174619
            Number of features: 0
            /start=174742
            /end=174619
            /accession=AL939120.1
            Per letter annotation for: secondary_structure
            Seq('CCAGGUAAGUCGCC-G-C--ACCG---------------GUCA-----------...GGA', SingleLetterAlphabet())
            GGAUCGCUGAACCCGAAAGGGGCGGGGGACCCAGAAAUGGGGCGAAUCUCUUCCGAAAGGAAGAGUAGGGUUACUCCUUCGACCCGAGCCCGUCAGCUAACCUCGCAAGCGUCCGAAGGAGAAUC

        """
        seq = seq.replace('-', '').upper()
        for s in self.io:
            seq_str = str(s.seq).replace('-', '').upper()
            if verbose:
                print(seq_str)
            if seq_str.find(seq) > -1 or seq.find(seq_str) > -1:
                print('Match:', s.id)
                print(s)
                print(seq)
        return s
        print('Not found')

    def find_seq_exact(self, seq, verbose=False):
        """Find seq (also subsequences) and reverse in the alignment.

        :param seq: string, seq, seq is upper()
        :param verbose: boolean, be verbose or not
        """
        seq = seq.replace('-', '').upper()
        for s in self.io:
            seq_str = str(s.seq).replace('-', '').upper()
            if verbose:
                print(seq_str)
            if seq_str == seq:
                print('Match:', s.id)
                print(s)
                print(seq)
                return s
        print('Not found')

    def get_clean_ss(self, ss):
        nss = ''
        for s in ss:
            nss += get_rfam_ss_notat_to_dot_bracket_notat(s)
        return nss

    def get_seq_ss(self, seq_id):  # seq,ss):
        s = self.get_seq(seq_id).seq
        # print seq, ss
        # new
        nseq = ''
        nss = ''
        for i, j in zip(seq, ss):
            if i != '-':  # gap
                # print i,j
                nseq += i
                nss += get_rfam_ss_notat_to_dot_bracket_notat(j)
        return nseq.strip(), nss.strip()

    def get_seq(self, seq_id):
        for s in self.io:
            if s.id == seq_id:
                return s
        raise Exception('Seq not found')

    def get_seq_with_name(self, seq_name):
        for s in self.io:
            if s.name == seq_name:
                return s
        raise Exception('Seq not found')

    def align_seq(self, seq):
        """Align seq to the alignment.

        Using self.rf.

        Args:

            seq (str): sequence, e.g. ``-GGAGAGUA-GAUGAUUCGCGUUAAGUGUGUGUGA-AUGGGAUGUC...``

        Returns:

           str: seq that can be inserted into alignemnt, ``.-.GG.AGAGUA-GAUGAUUCGCGUUA`` ! . -> -
        """
        seq = list(seq)
        seq.reverse()
        nseq = ''
        for n in self.rf:  # n nuclotide
            if n != '.':
                try:
                    j = seq.pop()
                except:
                    j = '.'
                nseq += j
            if n == '.':
                nseq += '.'  # + j
        return nseq.replace('.', '-')

    def __repr__(self):
        return (str(self.io))

    def trimmed_rf_and_ss(self):
        """Remove from RF and SS gaps.

        Returns:

          (str,str): trf, tss - new RF and SS

        """
        trf = ''
        tss = ''
        for r, s in zip(self.rf, self.ss_cons_std):
            if r not in ['-', '.']:
                trf += r
                tss += s
        return trf, tss

    def get_distances(self):
        """Get distances (seq identity) all-vs-all.

        With BioPython.

        blastn: ``Bad alphabet 'U' in sequence 'AE008922.1/409481-409568' at position '7'`` only for DNA?

        read more (also about matrix at <http://biopython.org/wiki/Phylo> and
        HTTP://biopython.org/DIST/docs/api/Bio.Phylo.TreeConstruction.DistanceCalculator-class.html
        """
        calculator = DistanceCalculator('identity')
        dm = calculator.get_distance(self.io)
        return dm

    def get_the_closest_seq_to_ref_seq(self, verbose=False):
        """

        Example::

            >>> a = RNAalignment("test_data/RF02221.stockholm.sto")
            >>> a.get_the_closest_seq_to_ref_seq()
            AF421314.1/431-344

        """
        self + RNASeq('ConSeq', self.rf, '')
        dist = self.get_distances()
        distConSeq = dist['ConSeq'][:-1]  # to remove ending 0, bc of distance to itself
        minimal = min(distConSeq)

        index = dist['ConSeq'].index(minimal)
        #id = dist.names[index]
        if verbose:
            print('dist:\n', str(dist))
            print('distConSeq:', dist['ConSeq'])
        return self[index]


class CMAlign():
    """CMAalign class around cmalign (of Inferal).

    cmalign - aligns  the RNA sequences in <seqfile> to the covariance model
    (CM) in <cmfile>.  The new alignment is output to stdout  in  Stockholm
    format.

    Example::

        cma = ra.CMAlign()
        cma.run_cmalign("ade_seq.fa", "RF00167.cm")
        seq = cma.get_seq()
        print 'cma hit  ', seq
        print 'seq      ', a.align_seq(seq)
        print 'a.rf     ', a.rf

        cmd cmalign -g RF00167.cm ade_seq.fa

        # STOCKHOLM 1.0
        #=GF AU Infernal 1.1.2

        ade          ----------------CGCUUCAUAUAAUCCUAAUGAUAUGGUUUGGGAGUUUCUACCAAGAG-CCUUAAA-CUCUUGAUUAUGAAGUGA------------
        #=GR ade PP  ................99*********************************************.*******.***************999............
        #=GC SS_cons :::::::::::::::::((((((((,,,<<<<<<<_______>>>>>>>,,,,,,,,<<<<<<<_______>>>>>>>,,))))))))::::::::::::::
        #=GC RF      aaaaaauaaaaaaaauucccuCgUAUAAucccgggAAUAUGGcccgggaGUUUCUACCaggcagCCGUAAAcugccuGACUAcGagggaaauuuuuuuuuuu
        //
        cma hit   ----------------CGCUUCAUAUAAUCCUAAUGAUAUGGUUUGGGAGUUUCUACCAAGAG-CCUUAAA-CUCUUGAUUAUGAAGUGA------------
        seq       ----------------CGCU-U-CAUAUAAUCCUAAUGAUAUGG-UUUGGGA-GUUUCUACCAAGAG-CC--UUAAA-CUCUU---GAUUAUG-AAGUGA-------------
        a.rf      aaaaaauaaaaaaaauuccc.u.CgUAUAAucccgggAAUAUGG.cccggga.GUUUCUACCaggcagCC..GUAAAcugccu...GACUAcG.agggaaauuuuuuuuuuu.

    Install http://eddylab.org/infernal/

    Cite: Nawrocki and S. R. Eddy, Infernal 1.1: 100-fold faster RNA homology searches, Bioinformatics 29:2933-2935 (2013). """

    def __init__(self, outputfn=None):
        """Use run_cmalign or load cmalign output from a file"""
        if outputfn:
            self.output = open(outputfn).read().strip().split('\n')

    def run_cmalign(self, seq, cm, verbose=True):
        """Run cmalign and process the result.

        :param seq: seq string
        :param cm: cm fn

        Run::

            $ cmalign RF01831.cm 4lvv.seq
            # STOCKHOLM 1.0
            #=GF AU Infernal 1.1.2

            4lvv         -GGAGAGUA-GAUGAUUCGCGUUAAGUGUGUGUGA-AUGGGAUGUCG-UCACACAACGAAGC---GAGA---GCGCGGUGAAUCAUU-GCAUCCGCUCCA
            #=GR 4lvv PP .********.******************9999998.***********.8999999******8...5555...8**************.************
            #=GC SS_cons (((((----(((((((((((,,,,,<<-<<<<<<<<___________>>>>>>>>>>,,,<<<<______>>>>,,,)))))))))))-------)))))
            #=GC RF      ggcaGAGUAGggugccgugcGUuAAGUGccggcgggAcGGGgaGUUGcccgccggACGAAgggcaaaauugcccGCGguacggcaccCGCAUcCgCugcc
            //

        .. warning :: requires cmalign to be set in your shell
        """
        tf = tempfile.NamedTemporaryFile(delete=False)
        tf.name += '.seq'

        with open(tf.name, 'w') as f:
            f.write('>target\n')
            f.write(seq + '\n')

        cmd = 'cmalign -g ' + cm + ' ' + tf.name  # global
        if verbose:
            print('cmd' + cmd)
        o = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        stdout = o.stdout.read().strip()
        stderr = o.stderr.read().strip()
        if verbose:
            print(stdout)
        self.output = stdout.split('\n')

    def get_gc_rf(self):
        """Get ``#=GC RF``.

        :var self.output: string
        """
        for l in self.output:
            if l.startswith('#=GC RF'):
                return l.replace('#=GC RF', '').strip()

    def get_seq(self):
        """
        :var self.output: output of cmalign, string
        """
        for l in self.output:
            if l.strip():
                if not l.startswith('#'):
                    #  4lvv         -GGAGAGUA-GAUGAU
                    return l.split()[1].strip()


def clean_seq_and_ss(seq, ss):
    nseq = ''
    nss = ''
    for i, j in zip(seq, ss):
        if i != '-':  # gap
            # print i,j
            nseq += i
            nss += get_rfam_ss_notat_to_dot_bracket_notat_per_char(j)
    return nseq.strip(), nss.strip()


def get_rfam_ss_notat_to_dot_bracket_notat_per_char(c):
    """Take (c)haracter and standardize ss (including pks in letter (AAaa) notation).

    .. warning:: DD DD will be treated as BB BB (<< >>) (it might be wrong!)"""
    if c in [',', '_', ':', '-']:
        return '.'
    if c == '<':
        return '('
    if c == '{':
        return '('
    if c == '}':
        return ')'
    if c == ']':
        return ')'
    if c == '[':
        return '('
    if c == '>':
        return ')'
    if c == '>':
        return ')'
    if c == 'A':
        return '['
    if c == 'a':
        return ']'
    if c == 'B':
        return '<'
    if c == 'b':
        return '>'
    if c == 'C':
        return '{'
    if c == 'c':
        return '}'
    # !!!!!!!!!!!!!!!!!!!!!!!!!!
    if c == 'D':
        return '<'
    if c == 'd':
        return '>'
    # !!!!!!!!!!!!!!!!!!!!!!!!!!
    return c


def get_rfam_ss_notat_to_dot_bracket_notat(ss):
    """Change all <<>> to "standard" dot bracket notation.

    Works also with pseudknots AA BB CC etc."""
    nss = ''
    for s in ss:
        ns = get_rfam_ss_notat_to_dot_bracket_notat_per_char(s)
        nss += ns
    return nss


def fasta2stokholm(fn):
    """Take a gapped fasta file and return an RNAalignment object.

    A fasta file should look like this::

        >AE009948.1/1094322-1094400
        UAC-U-UAUUUAUGCUGAGGAU--UGG--CUUAGC-GUCUCUACAAGACA-CC--GU-AA-UGUCU---AACAAUA-AGUA-
        ...
        >CP000721.1/2204691-2204778
        CAC-U-UAUAUAAAUCUGAUAAUAAGG-GUCGGAU-GUUUCUACCAAGCUACC--GUAAAUUGCUAUAGGACUAUA-AGUG-
        >SS_cons
        (((.(.((((...(((((((........))))))).........(((((((.........))))))...)..)))).)))).
        >SS_cons_pk
        .........................[[........................]].............................

    ``SS_cons_pk`` in optionally and is an extra line used to define a pseudoknot. You
    can also define second psedoknot as ``<<<...>>>`` and the third one with ``{{{ }}}``.

    :param fn: file
    :return: RNAalignment object
    """
    seqs = []
    s = None
    for l in open(fn):
        if l.startswith('>'):
            if s:
                seqs.append(s)
            s = RNASeq(l.replace('>', '').strip(), '')
        else:
            s.seq += l.strip()

    txt = ''
    for l in open(fn):
        if l.startswith('>'):
            id = '\n' + l.replace('>', '\n').strip() + ' '
            id = re.sub('ss_cons_pk', '#=GC SS_cons_pk', id, flags=re.IGNORECASE)
            id = re.sub('ss_cons', '#=GC SS_cons', id, flags=re.IGNORECASE)
            txt += id
        else:
            txt += l.strip()
    txt = txt.strip() + '\n'  # clean upfront \n and add tailing \n

    tf = tempfile.NamedTemporaryFile(delete=False)
    tf.name += '.stk'
    with open(tf.name, 'w') as f:
        f.write('# STOCKHOLM 1.0\n')
        f.write(txt)
        f.write('//\n')
    return RNAalignment(tf.name)


def fetch_stokholm(rfam_acc, dpath=None):
    """Fetch Stokholm file from Rfam.

    :param rfam_acc: str, Rfam accession number, eg. RF00028
    :param dpath: str or None, if None saves to current location, otherwise save to dpath folder

    .. warning :: urllib3 is required, pip install urllib3
    """
    import urllib3
    http = urllib3.PoolManager()
    # try:
    print(dpath)
    response = http.request('GET', url='http://rfam.xfam.org/family/' + rfam_acc.lower() +
                            '/alignment?acc=' + rfam_acc.lower() + '&format=stockholm&download=1')
    # except urllib3.HTTPError:
    #    raise Exception('The PDB does not exists: ' + pdb_id)
    txt = response.data
    if dpath is None:
        npath = rfam_acc + '.stk'
    else:
        npath = dpath + os.sep + rfam_acc + '.stk'
    print('downloading...' + npath)
    with open(npath, 'wb') as f:
        f.write(txt)
    print('ok')
    return rfam_acc + '.stk'


# def test_seq_multilines_alignment()
def test_alignment_with_pk():
    a = RNAalignment('test_data/RF00167.stockholm.sto')
    print(a.tail())
    print(a.ss_cons)
    print(a.ss_cons_pk)
    print(a.ss_cons_with_pk)

    print(a[[1, 2]])


# main
if __name__ == '__main__':
    ## a = RNAalignment('test_data/RF00167.stockholm.sto')
    # print a.get_shift_seq_in_align()
    # print a.map_seq_on_align('CP000721.1/2204691-2204778', [5,6,8])
    # print a.map_seq_on_seq('AAML04000013.1/228868-228953', 'CP000721.1/2204691-2204778', [4,5,6])

    # print(record.letter_annotations['secondary_structure'])
    ## seq = a.get_seq('AAML04000013.1/228868-228953')
    # print seq.seq
    # print seq.ss
    # print a.ss_cons

    # print 'x'
    # for s in a.io:
    # print s.seq
    # print s.ss_clean #letter_annotations['secondary_structure']

    # for s in a.io:
    # print s.seq_nogaps
    # print s.ss_nogaps

    # a.write('test_output/out.sto')

    # a = RNAalignment("/home/magnus/work/rna-evo/rp12/seq/RF00379.stockholm.stk")##_rm85.stk')
    # print a.get_seq_ss("AL939120.1/174742-174619")
    # subset = a.subset(["AL939120.1/174742-174619",
    # "rp12bsubtilis",
    # "AAWL01000001.1/57092-56953",
    # "CP000612.1/87012-87130",
    # "BA000028.3/1706087-1706245"])
    # % cat /var/folders/yc/ssr9692s5fzf7k165grnhpk80000gp/T/tmpTzPenx
    # for s in subset:
    # print s.seq
    # subset.remove_empty_columns()
    # subset.write('/home/magnus/out2.stk')
    # for s in subset:
    # print s.seq

    # print 'subset.ss_cons_std:', subset.ss_cons_std

    # a = RNAalignment("/home/magnus/work/rna-evo/rp12/seq/RF00379.stockholm.stk")##_rm85.stk')
    # a.ss_cons = "::::::::::{{{{,,,<<<_..."
    # print a.ss_cons
    # print a.ss_cons_std
    # print get_rfam_ss_notat_to_dot_bracket_notat(subset.ss_cons)

    ## a.ss_cons_std = 'test of setter'
    # print a._ss_cons_std

    # pass
    # slice

    ## slices = a.cols[0:10]
    # for s in a:
    # print s.seq
    # add pk

    # for s in slices:
    # print s, s.seq, s.ss

    #a = fasta2stokholm('test_output/ade_gapped.fa')
    # print a

    #a = RNAalignment('test_data/RF00001.blocked.stk')
    # print a
    # print a.ss_cons
    # a.plot('rchie.png')

    # a = RNAalignment("test_data/RF02221.stockholm.sto")
    ## a = RNAalignment('test_data/test_data/RF02221.stockholm.sto')
    ## a + RNASeq('ConSeq', '-A-GU-AGAGUA-GGUCUUAUACGUAA-----------------AGUG-UCAUCGGA-U-GGGGAGACUUCCGGUGAACGAA-G-G-----------------------------GUUA---------------------------CCGCGUUAUAUGAC-C-GCUUCCG-CUA-C-U-','')
    ## dist = a.get_distances()
    # distConSeq = dist['ConSeq'][:-1]  # to remove ending 0, bc of distance to itself
    ## minimal = min(distConSeq)
    ## index = dist['ConSeq'].index(minimal)
    # print(dist.names[index])

    a = RNAalignment("test_data/dist_test2.stk")
    rep = a.get_the_closest_seq_to_ref_seq()
    rep.remove_gaps()
    print(rep)
    print(rep.ss)
    print(rep.seq)

    # a.write('tmp.stk')
    # s.remove_gaps()
    # print(s.seq)
    # print(s.ss)

    import doctest
    doctest.testmod()



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""rna_clanstix - a tool for visualizing RNA 3D structures based on pairwise structural similarity with Clans.

We hacked Clans thus instead of BLAST-based distances between sequences, you can analyze distances between structures described as p-values of rmsd (based on the method from the Dokholyan lab.)

Running Clans:
To run CLANS you need to have Java 1.4 or better installed (java can be downloaded HERE). For full functionality you will also need the NCBI BLAST,PSI-BLAST and formatdb executables (NCBI). For command line parameters and basic help please refer to the README file.
(source: http://www.eb.tuebingen.mpg.de/research/departments/protein-evolution/software/clans.html)

.. image:: ../../rna_tools/tools/clanstix/doc/yndSrLTb7l.gif

The RMSDs between structures are converted into p-values based on the method from the Dokholyan lab.

Color groups
---------------------------------------
You can color your groups:

.. image:: ../../rna_tools/tools/clanstix/doc/rna_clanstix.png

To get colors, run a cmd like this::

   rna_clastix.py rnapz17_matrix_farfar_HelSeedCst.txt --groups 20:seq1+20+20+20+20+20+20:seq10

where with the ``+`` sign you separate groups. Each group has to have a number of structures. Optionally it can have a name, e.g., ``20:seq1``, use ``:`` as a separator. If a provided name is ``native`` then this group will be shown as starts.

Get inspiration for more colors (http://www.rapidtables.com/web/color/RGB_Color.htm)

How to use ClanstixRNA?
----------------------------------------

1. Get a matrix of distances, save it as e.g. matrix.txt (see Comment below)
2. run ClanstixRNA on this matrix to get an input file to Clans (e.g. clans_rna.txt)::

     rna_clanstix.py test_data/matrix.txt > clans_run.txt

3. open CLANS and click File -> Load run and load clans_run.txt
4. You're done! :-)

Comment: To get this matrix you can use for example another tool from the rna-pdb-tools packages::

     rna_calc_rmsd_all_vs_all.py -i rp18 -o rp18_rmsd.csv
     rna_clastix.py --groups 1:native+5:3dRNA+
           5:Chen+3:Dokh+5:Feng+5:LeeASModel+
           5:Lee+5:RNAComposer+10:RW3D+5:Rhiju+
           1:YagoubAli+3:SimRNA  rp18_rmsd.csv clans.in

     rna_clastix.py --groups 100+100+100+100+100+100+100+100+100+100+1:native  rp18_rmsd.csv

where ``rp18`` is a folder with structure and ``rp18_rmsd.csv`` is a matrix of all-vs-all rmsds.

.. image:: ../../rna_tools/tools/clanstix/doc/rp18_clanstix.png

Hajdin, C. E., Ding, F., Dokholyan, N. V, & Weeks, K. M. (2010). On the significance of an RNA tertiary structure prediction. RNA (New York, N.Y.), 16(7), 1340–9. doi:10.1261/rna.1837410

An output of this tool can be viewed using CLANS.

Frickey, T., & Lupas, A. (2004). CLANS: a Java application for visualizing protein families based on pairwise similarity. Bioinformatics (Oxford, England), 20(18), 3702–4. doi:10.1093/bioinformatics/bth444
"""

import argparse
import rna_tools.tools.rmsd_signif.rnastruc_pred_signif as pv
import numpy as np
import math
import logging
import time


logging.basicConfig(level=logging.INFO,
                format='%(message)s',
                datefmt='%m-%d %H:%M',
                filename='rna_clanstix.log',
                filemode='w')

console = logging.StreamHandler()
console.setLevel(logging.INFO)
formatter = logging.Formatter('%(message)s')
console.setFormatter(formatter)
logging.getLogger('').addHandler(console)

log = logging.getLogger()
log.setLevel(logging.INFO)


class RNAStructClans:
    """Clans run.

    Usage::

        >>> f = open('matrix.txt')
        >>> ids = f.readline().replace('#','').split()
        >>> c = RNAStructClans(n=len(ids)) # 200?
        >>> c.add_ids(ids)
        >>> c.dist_from_matrix(f)
        >>> print(c.txt)
    """

    def __init__(self, n=10):
        self.n = n
        self.comment = ''
        self.txt = """sequences=%i
<param>
maxmove=0.1
pval=1.0E-15
usescval=false
complexatt=true
cooling=1.0
currcool=1.0
attfactor=10.0
attvalpow=1
repfactor=10.0
repvalpow=1
dampening=1.0
minattract=1.0
cluster2d=false
blastpath=blastall -p blastp -I T -T T
formatdbpath=''
showinfo=false
zoom=1.0
dotsize=1
ovalsize=10
groupsize=4
usefoldchange=false
avgfoldchange=false
colorcutoffs=0.0;0.1;0.2;0.3;0.4;0.5;0.6;0.7;0.8;0.9;
colorarr=(230;230;230):(207;207;207):(184;184;184):(161;161;161):(138;138;138):(115;115;115):(92;92;92):(69;69;69):(46;46;46):(23;23;23):
</param>""" % n

    def add_ids(self, ids):
        t = '\n<seq>\n'
        for i in ids:
            t += '>' + i + '\n'
            t += 'X' + '\n'
        t += '</seq>\n'
        if len(ids) != self.n:
            # print 'n != ids'
            raise Exception('n != ids')
        self.txt += t

    def dist_from_matrix(self, lines, matrix=0, use_pv=False):
        # return '' # for some hardcore debugging ;-)
        t = '\n<hsp>\n'
        c = 0
        c2 = 0
        for l in lines:
            for rmsd in l.split():
                if c != c2:
                    if use_pv:
                        dist = pv.get_p_value(rmsd, 1 * 38)[0]  # r.get_rmsd_to(r2), 3)
                    else:
                        # 1e-06 10-1 = 9 10-10 0
                        dist = '1.0E-' + str(int(math.floor(matrix.max()) - int(float(rmsd))))
                    t += str(c) + ' ' + str(c2) + ':' + str(dist) + '\n'
                c2 += 1
            c2 = 0
            c += 1

        t += '</hsp>\n'

        self.comment = '# max: %f min (non-zero): %f\n' % (math.ceil(matrix.max()), matrix[matrix>0].min())
        for i in range(1,20):
            self.comment += '# connected points with RMSD lower than %iA 1.0E-%i\n' % (i, math.ceil(matrix.max()) - i)
        # 1E-11 = 0
        # 1E-10 = 1-0
        # 1E-9 = 2-1
        self.txt += t


def get_parser():
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('matrixfn', help="matrix")

    parser.add_argument('--groups-auto', help="groups automatically <3", type=int, default=10)

    parser.add_argument('--groups', help="groups, at the moment up to 7 groups can be handle"
                        "--groups 1:native+100:zmp+100:zaa+100:zbaa+100:zcp+100:znc"
                        "--groups 1:native+100:nats+100:hom1+100:hom2+100:hom3+100:hom4"
                        "native will light green"
                        "zmp will be forest green"
                        "(easy to be changed in the future)")
    parser.add_argument('--pvalue', action='store_true',
                        help="")
    parser.add_argument('output', help="input file for clans")
    return parser


# main
if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    f = open(args.matrixfn)
    ids = f.readline().replace('#', '').split()
    # print ids
    # print len(ids)
    # get max
    logging.info(time.strftime("%Y-%m-%d %H:%M:%S"))

    matrix = np.loadtxt(args.matrixfn)

    c = RNAStructClans(n=len(ids))  # 200?
    c.add_ids(ids)
    c.dist_from_matrix(f, matrix, args.pvalue)

    #
    # DEFINE GROUPS
    #
    # 1+20+20+20+20+20
    seqgroups = ''
    #colors = ['0;255;102;255', # ligthgreen 1
    #          '0;102;0;255', # forest 2
    colors = [
              '255;102;102;255', # red 3
              '51;51;255;255', # blue 4
              '0;255;255;255', # light blue +1
              '180;38;223;255', # violet 5
              '64;64;64;255', # grey 6
              '255;128;0;255', # orange 7
              '128;0;128;255', # purple 8
              '0;128;128;255', # Teal 9
              '128;0;0;255', # maroon 10
              '0;255;255;255',  # cyjan
              '240;230;140;255', #khaki
              '210;105;30;255', # chocolate
               ]

    args_groups = args.groups
    if args.groups_auto:
        from collections import OrderedDict
        # collect groups
        #groups = []
        groups = OrderedDict()
        for i in ids:
            group_name = i[:args.groups_auto]
            if group_name in groups:
                groups[group_name] += 1
            else:
                groups[group_name] = 1
        groups_str = ''
        for g in groups:
            groups_str += str(groups[g]) + ':' + g + '+'

        args_groups = groups_str[:-1]
        #print(groups_str)
        # change this to get 1:hccp+10:zmp+10:xrt

    if args_groups:
        # this is a manual way how to do it
        groups = args_groups.split('+')
        seqgroups = '<seqgroups>\n'
        curr_number = 0
        for index, group in enumerate(groups):
            # parse groups
            # type and size will be changed for native
            size = 8
            dottype = 0
            color = '' # use for diff color selection if tar
            if ':' in group:
                nstruc, name = group.split(':')
                if name == 'native':
                    dottype = 8
                    size = 20
                    color = '0;128;128;255' # olive

                if 'solution' in name:
                    dottype = 8
                    size = 20
                    color = '0;128;128;255' # olive

                if 'tar' in name and 'tar_min' not in name: # simrna
                    dottype = 7
                    size = 20
                    color = '0;255;102;255' # ligthgreen 1

                if 'tar_min' in name:
                    dottype = 9
                    size = 20
                    color = '0;102;0;255' # forest 2
            else:
                nstruc = group
                name = 'foo'

            # craft seqgroups
            seqgroups += "name=%s\n" % name
            seqgroups += "type=%i\n" % dottype
            # color hack
            if color:  # this color is fixed for native right now
                seqgroups += "color=%s\n" % color
            else:
                seqgroups += "color=%s\n" % colors[index]
            # color hack
            seqgroups += "size=%i\n" % size
            seqgroups += "hide=0\n"
            # get numbers - convert nstruc into numbers in Clans format 0;1; etc.
            # remember: it starts from 0
            # --groups 1:hccp+10:zmp+10:xrt
            # hccp
            # [0]
            # zmp
            # [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
            # xrt
            # [11, 12, 13, 14, 15, 16, 17, 18, 19, 20]
            numbers = range(curr_number, curr_number + int(nstruc))  # 0, 1
            curr_number = curr_number + int(nstruc)
            seqgroups += "numbers=%s;\n" % ';'.join([str(number) for number in numbers])
        seqgroups += '</seqgroups>\n'

    with open(args.output, 'w') as f:
        f.write(c.txt)
        f.write(seqgroups)
        f.write(c.comment)
    print(c.comment)
    logging.info(time.strftime("%Y-%m-%d %H:%M:%S"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
r"""rna_rosetta_run.py - prepare & run ROSETTA simulations

Based on C. Y. Cheng, F. C. Chou, and R. Das, Modeling complex RNA tertiary folds with Rosetta, 1st ed., vol. 553. Elsevier Inc., 2015.
http: // www.sciencedirect.com / science / article / pii / S0076687914000524

The script makes(1) a folder for you job, with seq.fa, ss.fa, input file is copied as input.fa to the folder(2) make helices(3) prepare rosetta input files(4) sends jobs to the cluster.

The header is take from the fast file(`` > /header / ``) not from the filename of your Fasta file.

I discovered this::

     qstat -xml | tr '\n' ' ' | sed 's#<job_list[^>]*>#\n#g' \
>   | sed 's#<[^>]*>##g' | grep " " | column -t

(https://stackoverflow.com/questions/26104116/qstat-and-long-job-names) so there is now need to shorted my job ids.

Helix
-------------------------------------------------------

Run::

    rna_rosetta_run.py -i -e -r -g -c 200 cp20.fa

`-i`::

    # prepare a folder for a run
    >cp20
    AUUAUCAAGAAUCUCAAAGAGAGAUAGCAACCUGCAAUAACGAGCAAGGUGCUAAAAUAGAUAAGCCAAAUUCAAUUGGAAAAAAUGUUAA
    .(((((....(((((.....)))))(((..(((((..[[[[..)).))).)))......))))).((((......)))).......]]]].

    [peyote2] ~ rna_rosetta_run.py -i cp20.fa
    run rosetta for:
    cp20
    AUUAUCAAGAAUCUCAAAGAGAGAUAGCAACCUGCAAUAACGAGCAAGGUGCUAAAAUAGAUAAGCCAAAUUCAAUUGGAAAAAAUGUUAA
    .(((((....(((((.....)))))(((..(((((..[[[[..)).))).)))......))))).((((......)))).......]]]].
    /home / magnus // cp20 / created
    Seq & ss created

Troubleshooting.

If one of the helices is missing you will get::

    IOError: [Errno 2] No such file or directory: 'helix1.out'
    rosetta_submit.py README_FARFAR o 500 100 taf
    Could not find:  README_FARFAR

and the problem was a1 and g8 pairing::

    outputting command line to:  helix0.RUN  # previous helix #0
    Sequence:  AUGG CCGG
    Secstruc:  (((())))
    Not complementary at positions a1 and g8!
    Sequence:  GUGGG CCCAU
    Secstruc:  ((((()))))

    Writing to fasta file:  helix2.fasta  # next helix #2

My case with a modeling of rp12

    Sequence:  cc gc
    Secstruc:  (())
    Not complementary at positions 1 and 4!

edit the secondary structure, run the program with -i(init, to overwrite seq.fa, ss.fa) and then it works.


Notes::

   rp17hc 6 characters

"""
from __future__ import print_function
import argparse
import textwrap
import os
import glob
import subprocess
import shutil
import math

import os
import sys

try:
    from rna_tools.rna_tools_config import RNA_ROSETTA_RUN_ROOT_DIR_MODELING
except:
    RNA_ROSETTA_RUN_ROOT_DIR_MODELING = ''
    print ('Set up rna_rosetta_run_root_dir_for_modeling in rpt_config_local.py')


class CustomFormatter(argparse.ArgumentDefaultsHelpFormatter, argparse.RawTextHelpFormatter):
    pass


def get_parser():
    parser = argparse.ArgumentParser(description=__doc__,
                                     formatter_class=CustomFormatter)

    parser.add_argument('-i', '--init', help='prepare _folder with seq and ss',
                        action='store_true')
    parser.add_argument('-e', '--helices', help='produce h(E)lices',
                        action='store_true')
    parser.add_argument('-r', '--rosetta', help='prepare rosetta files (still you need `go` to send jobs to a cluster)',
                        action='store_true')
    parser.add_argument('-g', '--go', help=go.__doc__,
                        action='store_true')
    parser.add_argument('-m', '--motif', help="include a motif file, e.g. -s E-loop_1q9a_mutated_no_flanks_renumber.pdb")
    parser.add_argument('-n', '--nstruc', help="# of structures you want to get",
                        default=10000, type=int)

    parser.add_argument('-c', '--cpus', help='# of cpus to be used', default=200,
                        type=int)

    parser.add_argument('--sandbox', help="where to run it (default: RNA_ROSETTA_RUN_ROOT_DIR_MODELING",
                        default=RNA_ROSETTA_RUN_ROOT_DIR_MODELING)

    parser.add_argument('file', help=textwrap.dedent(
        """file: \n > a04\nUAUAACAUAUAAUUUUGACAAUAUGGGUCAUAAGUUUCUACCGGAAUACCGUAAAUAUUCUGACUAUGUAUA\n((((.((((...((.((((.......)))).))........(.(((((.......))))).)..))))))))"""))
    return parser


def prepare_folder(args, header, seq, ss, path):
    """Make folder for you job, with seq.fa, ss.fa, input file is copied as input.fa to the folder.

    For ss lowercase is needed when used with motifs, otherwise::

        [peyote2] aa20$ rna_rosetta_run.py -r -m E-loop_1q9a_mutated_no_flanks_renumber_for_acy20.pdb ~/aa20.fa
        2019-05-03 21:31:30,842 rpt_config_local.py::<module>::rpt_config_local loading...
        run rosetta for:
        aa20
        UACGUUCAUCAUCCGUUUGGAUGACGGAAGUAAGCGAAAGCUGAAGGAACGCAUG
        ..(((((.((((((....))))))..[.....(((....)))....)))))]...
        rna_denovo_setup.py -fasta seq.fa -secstruct_file ss.fa -cycles 20000 -no_minimize -nstruct 50  -s E-loop_1q9a_mutated_no_flanks_renumber_for_acy20.pdb  -silent helix0.out helix1.out helix2.out -input_silent_res 3-7 47-51 9-14 19-24 33-35 40-42

        Sequence:  UACGUUCAUCAUCCGUUUGGAUGACGGAAGUAAGCGAAAGCUGAAGGAACGCAUG
        Secstruc:  ..(((((.((((((....))))))..[.....(((....)))....)))))]...
        aaguagaag
        AAGUAGAAG
        Traceback (most recent call last):
          File "/home/magnus/opt/rosetta_src_2016.13.58602_bundle/tools/rna_tools/bin//rna_denovo_setup.py", line 170, in <module>
            raise ValueError('The sequence in %s does not match input sequence!!' % pdb)
        ValueError: The sequence in E-loop_1q9a_mutated_no_flanks_renumber_for_acy20.pdb does not match input sequence!!
        rosetta_submit.py README_FARFAR o 200 100 aa20_
        Could not find:  README_FARFAR
    """
    d = path
    try:
        os.mkdir(d)
        print(d, 'created')
    except OSError:
        print(d, 'created is already created')
        pass

    with open(d + "seq.fa", "w") as f:
        f.write(header + '\n')
        f.write(seq)

    with open(d + "ss.fa", "w") as f:
        f.write(ss.lower())
    print('Seq & ss created')
    shutil.copyfile(args.file, d + 'input.fa')


def prepare_helices():
    """Make helices(wrapper around 'helix_preassemble_setup.py')

    .. warning:: I think multiprocessing of helixX.run does not work."""
    # run helix_p..
    cmd = 'helix_preassemble_setup.py -secstruct ss.fa -fasta seq.fa'
    os.system(cmd)

    # find all helix
    helix_runs = glob.glob('*RUN')
    print(helix_runs)

    f = open('HRUNS', 'w')
    for h in helix_runs:
        f.write(open(h).read().strip() + ' & \n')
    f.close()

    # does not work (!)
    # os.system('chmod +x CMDLINES')
    # os.system('./CMDLINES')
    # ./CMDLINES: 2: source: not found

    # os.system runs multiprocessing, but does not wait for the rest of the program
    # hmm... it does not wait because I used & ???
    # this works in multirpocessing mode but it does not wait for `-r` !!! so if your run -e only it's OK.
    # don't combile -e with -r because making helices will not wait to run -r (!) and  you will get an error
    # and only helices made and then the program will finish
    if False:
        os.system('bash HRUNS')
    else:
        p = subprocess.Popen(open('HRUNS').read(),
                             shell=True, stderr=subprocess.PIPE)
        p.wait()
        stderr = p.stderr.read().strip()
        if stderr:
            print(stderr)


def prepare_rosetta(header, cpus, motif, nstruc):
    """Prepare ROSETTA using rna_denovo_setup.py

    cpus is used to calc nstruc per job to get 10k structures per full run::

    Args:

      nstruc(int): how many structures you want to obtain
      nstruct = int(math.floor(20000 / cpus))
      motif: motif file; e.g., -s E-loop_1q9a_mutated_no_flanks_renumber.pdb
      50 (nstruc) = 10k / 200 (cpus)

    """
    # get list line
    helices = open('CMDLINES').readlines()[-1].replace('#', '')
    njobs = cpus  # 500
    nstruct = int(math.floor(nstruc / cpus))  # 20000/500 -> 40
    if motif:
        cmd_motif = ' -s ' + motif
    else:
        cmd_motif = ''
    cmd = 'rna_denovo_setup.py -fasta seq.fa -secstruct_file ss.fa -cycles 20000 -no_minimize -nstruct ' + \
        str(nstruct) + ' ' + cmd_motif + ' ' + helices
    print(cmd)
    os.system(cmd)
    # change to 50 per job (!)
    # 50 * 100 = 10k ?
    # dont change this 100 (!) it might not run on peyote2 with values like 99999 !
    # cmd is
    # rna_tools/bin//rosetta_submit.py <text file with rosetta command> <outdir> <# jobs> <# hours> <job name>
    # manually: [peyote2] a22$ rosetta_submit.py README_FARFAR o 200 100 a22_
    cmd = 'rosetta_submit.py README_FARFAR o ' + \
        str(njobs) + ' 100 ' + header + '_'
    print(cmd)
    os.system(cmd)


def go():
    """send jobs to a cluster(run qsubMINI)"""
    os.system('chmod +x ./qsubMINI')
    os.system('./qsubMINI')


def main():
    """Pipeline for modeling RNA"""
    args = get_parser().parse_args()

    if args.file:
        f = open(args.file)
        header = f.readline().replace('>', '').replace(' ', '').strip()
        seq = f.readline().strip()
        ss = f.readline().strip()
        cpus = int(args.cpus)
        print('run rosetta for:')
        print(header)

        print(seq)
        print(ss)

        if RNA_ROSETTA_RUN_ROOT_DIR_MODELING.strip() == '':
            print('Set RNA_ROSETTA_RUN_ROOT_DIR_MODELING in your rpt_config file.')
            return

        path = args.sandbox + os.sep + header + \
            os.sep  # RNA_ROSETTA_RUN_ROOT_DIR_MODELING
        curr = os.getcwd()
        if args.init:
            prepare_folder(args, header, seq, ss, path)
        try:
            os.chdir(path)
        except OSError:
            print('You have to make a folder first! use --init')
            sys.exit(1)

        if args.helices:
            prepare_helices()
        if args.rosetta:
            prepare_rosetta(header, cpus, args.motif, args.nstruc)
        if args.go:
            go()

        os.chdir(curr)


# main
if __name__ == '__main__':
    main()



#!/usr/bin/env python

from __future__ import print_function
import os
import sys

from rna_tools.BlastPDB import BlastPDB
from rna_tools.RfamSearch import RNASequence, RfamSearch
from rna_tools.rna_tools_lib import RNAStructure
from rna_tools.rna_tools_lib import edit_pdb, add_header, get_version, \
                          collapsed_view, fetch, fetch_ba, replace_chain, RNAStructure, \
                          select_pdb_fragment, sort_strings
from rna_tools.tools.rna_x3dna.rna_x3dna import x3DNA


def test_blastpdb():
    p = BlastPDB('GGGUCAGGCCGGCGAAAGUCGCCACAGUUUGGGGAAAGCUGUGCAGCCUGUAACCCCCCCACGAAAGUGGG')
    p.search()
    assert p.result.startswith('<HTML') is True


def test_rfamsearch():
    seq = RNASequence("GGCGCGGCACCGUCCGCGGAACAAACGG")
    rs = RfamSearch()
    hit = rs.cmscan(seq)
    print(hit)


def test_rnastructre():
    pass

if __name__ == '__main__':
    # get version
    version = os.path.basename(os.path.dirname(os.path.abspath(__file__))), get_version(__file__)
    version = version[1].strip()

    print('- Python:', sys.version.replace('\n', ''))

    print('- rna-tools:', version)

    print('- RNA_TOOLS_PATH set to ', os.environ['RNA_TOOLS_PATH'])

    print('- See full list of tools <https://github.com/mmagnus/rna-tools/blob/master/rna-tools-index.csv')
    
    print('Seems OK')




    



import logging
logger = logging.getLogger()
handler = logging.StreamHandler()
logger.addHandler(handler)
formatter = logging.Formatter(
    '%(asctime)-15s %(filename)s::%(funcName)s::%(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)
logger.setLevel(logging.INFO)



#!/usr/bin/python

from SingleLineUtils import get_res_code, get_res_num, get_atom_code, \
    set_atom_code
import PDBFile
import os

if __name__ == '__main__':
    fn = '1xjrA_M1.pdb'
    pdb_file = PDBFile.PDBFile(pdb_path='test' + os.sep + fn)
    print((pdb_file.pdb_lines[0]))
    pdb_file.pedantic_pdb()
    print((pdb_file.fixes))
    print((type(pdb_file.pdb_lines)))
    print(('14:', pdb_file.pdb_lines[0]))
    pdb_file.remove_non_atoms()
    print((pdb_file.pdb_lines[0]))
    pdb_file.check_and_add_P_at_start()
    pdb_file.check_and_add_P_at_start()
    print((pdb_file.fixes))
    pdb_file.save('tmp.pdb')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_alignment_r2r.py - a simple wrapper for r2r
R2R
Software to speed the depiction of aesthetic consensus RNA secondary structures
Zasha Weinberg

Z. Weinberg and R. R. Breaker. R2R—software to speed the depiction of aesthetic consensus RNA secondary structures. BMC Bioinformatics, 21:3, 2011.

Example::

    rna_alignment_r2r.py Intron_gpII-A_BactA.stk

What's next: the script will be integrated into RNAalignment class
"""

from __future__ import print_function
import rna_tools.tools.rna_alignment.rna_alignment as ra
import argparse
import subprocess


def get_parser():
    parser = argparse.ArgumentParser(description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('file', help="fasta seq")
    parser.add_argument("--svg", default=True,
                        action="store_true", help="output also a svg file")
    parser.add_argument("-v", "--verbose", default=True,
                        action="store_true", help="be verbose")
    return parser


def exe(cmd, verbose):
    if verbose:
        print(cmd)
    o = subprocess.Popen(
        cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out = o.stdout.read().strip().decode()
    err = o.stderr.read().strip().decode()
    return out, err


def r2r(alignment, svg, verbose):
    cons =  alignment + '.cons'
    pdf =  alignment.replace('.stk', '').replace('.sto', '') + '.pdf'
    svg =  alignment.replace('.stk', '').replace('.sto', '') + '.svg'
    cmd = 'r2r --GSC-weighted-consensus ' + alignment + ' ' + cons + ' 3 0.97 0.9 0.75 4 0.97 0.9 0.75 0.5 0.1'
    out, err = exe(cmd, verbose)
    if verbose: print(out)
    if err:
        print(err)

    cmd = 'r2r ' + cons + ' ' + pdf
    out, err = exe(cmd, verbose)
    if err:
        print(err)
    if verbose: print(out)

    if svg:
        cmd = 'r2r ' + cons + ' ' + svg
        out, err = exe(cmd, verbose)
        if err:
            print(err)
        if verbose: print(out)


if __name__ == '__main__':
    # for debugging #
    #fn = 'test_data/RF00167.stockholm.sto'
    #ids = ['AL591975.1/251136-251218','CP000721.1/2204691-2204778']#ACCL02000010.1/116901-116991']
    parser = get_parser()
    args = parser.parse_args()

    r2r(args.file, args.svg, args.verbose)



from numpy import array, sqrt

from Bio import PDB
from Bio.PDB.PDBIO import Select
from Bio.PDB import PDBIO, Superimposer

#from Bio.SVDSuperimposer import SVDSuperimposer
#from SVDSuperimposerDontMove import SVDSuperimposer
import numpy

class BasePair:
    """BasePair Class

    methods:
    - get_coord()
    - __sub__() - calc distance between two base pairs

    attributs:
    - coord - centroid of base pair"""
    def __init__(self, a,b,struct=None):
        """a,b, struct biopython residue

        struct is needed for saving a pdb file (default is None)"""
        self.a = a
        self.b = b
        self.struct = struct
        self.coord = None
        self.name = 'bp-' + a.get_resname().strip() + str(a.get_id()[1]) + '-' + b.get_resname().strip() + str(b.get_id()[1])
        self.coord = self.calc_coord()
        self.atoms_ab = self.get_atoms_ab()
        self.atoms_ba = self.get_atoms_ba()

    def get_name(self):
        return self.name
    def calc_coord(self):
        """Return coord of base pair

        exclude sugar and posphate and hydrogens"""
        centroid=PDB.Vector(0,0,0)
        counter=0.0
        #exclude = ["O5'","C5'", "C4'", "O4'", "C3'", "O3'", "C2'", "O2'", "P", "OP2", "OP1", "OP3","C1'"] # 
        exclude = []
        for atom in self.a:
            if atom.name not in exclude:
                if not atom.name.startswith('H'): 
                    centroid=centroid+atom.get_vector()
                    counter+=1
        for atom in self.b:
            if atom.name not in exclude:
               if not atom.name.startswith('H'): 
                    centroid=centroid+atom.get_vector()
                    counter+=1

        centroid=centroid/counter
        self.coord = numpy.array((tuple(centroid)), 'f')
        return self.coord

    def __sub__(self, other):
        """Calc distance between two base pairs"""
        diff = self.coord - other.coord 
        return numpy.sqrt(numpy.dot(diff, diff)) 
    def __str__(self):
        return self.name

    def overlap_with(self, other):
        diff = self.coord - other.coord         
        self.new_coord = self.coord - diff

        self.a_bk = self.a.copy()
        self.b_bk = self.b.copy()

        for atom in self.a:
            atom.set_coord(atom.get_coord() - diff)
        for atom in self.b:
            atom.set_coord(atom.get_coord() - diff)

    def save(self):
        if not self.struct:
            raise Exception('self.struct was not defined! Can not save a pdb!')
        class BpSelect(Select):
            def accept_residue(self, residue):
                if residue.get_id()[1] == 1 or residue.get_id()[1] == 43:
                    return 1
                else:
                    return 0

        io = PDBIO()
        io.set_structure(self.struct)
        fn = self.name + '.pdb'
        io.save(fn, BpSelect())
        return 'Saved to: %s ' % fn

    def get_atoms_ab(self):
        """Get 18 atoms per base pair.

        18
        [<Atom C5'>, <Atom C4'>, <Atom O4'>, <Atom C3'>, <Atom O3'>, <Atom C1'>, <Atom C2'>, <Atom O2'>, <Atom N9>, <Atom C5'>, <Atom C4'>, <Atom O4'>, <Atom C3'>, <Atom O3'>, <Atom C1'>, <Atom C2'>, <Atom O2'>, <Atom N1>]"""
        self.atoms = []

        #for atom in self.a:
        #    atoms.append(atom)
        #for atom in self.a:
        #    atoms.append(atom) 
        include = ["C5'", "C4'", "O4'", "C3'", "O3'", "C2'", "O2'", "C1'"]# "OP2", "OP1", "OP3",  "P" "O5'" # <- first pair does not have them
        #self.atoms.extend([self.a['C3\''], self.b['C3\'']])

        for at in self.b:
            if at.name in include:
                self.atoms.append(at)
            if self.b.resname.strip() in ['G', 'A'] and at.name == "N9":
                self.atoms.append(at)
            if self.b.resname.strip() in ['U', 'C'] and at.name == "N1":
                self.atoms.append(at)

        for at in self.a:
            if at.name in include:
                self.atoms.append(at)
            if self.a.resname.strip() in ['G', 'A'] and at.name == "N9":
                self.atoms.append(at)
            if self.a.resname.strip() in ['U', 'C'] and at.name == "N1":
                self.atoms.append(at)

        return self.atoms

    def get_atoms_ba(self):
        """Return 18 atoms per base pair.

        18
        [<Atom C5'>, <Atom C4'>, <Atom O4'>, <Atom C3'>, <Atom O3'>, <Atom C1'>, <Atom C2'>, <Atom O2'>, <Atom N9>, <Atom C5'>, <Atom C4'>, <Atom O4'>, <Atom C3'>, <Atom O3'>, <Atom C1'>, <Atom C2'>, <Atom O2'>, <Atom N1>]

        print len(coords1) --> 18!"""
        self.atoms = []

        #for atom in self.a:
        #    atoms.append(atom)
        #for atom in self.a:
        #    atoms.append(atom) 
        include = ["C5'", "C4'", "O4'", "C3'", "O3'", "C2'", "O2'", "C1'"]# "OP2", "OP1", "OP3",  "P" "O5'" # <- first pair does not have them
        #self.atoms.extend([self.a['C3\''], self.b['C3\'']])

        for at in self.a:
            if at.name in include:
                self.atoms.append(at)
            if self.a.resname.strip() in ['G', 'A'] and at.name == "N9":
                self.atoms.append(at)
            if self.a.resname.strip() in ['U', 'C'] and at.name == "N1":
                self.atoms.append(at)

        for at in self.b:
            if at.name in include:
                self.atoms.append(at)
            if self.b.resname.strip() in ['G', 'A'] and at.name == "N9":
                self.atoms.append(at)
            if self.b.resname.strip() in ['U', 'C'] and at.name == "N1":
                self.atoms.append(at)

        return self.atoms

    def calc_rmsd_to(self, other):
        """Return rmsd to other bp (it calc two rmsd a-b and b-a and return min)."""
        #r1
        coords1 = array(tuple([tuple(atom.get_vector()) for atom in self.atoms_ab]), 'f')
        coords2 = array(tuple([tuple(atom.get_vector()) for atom in other.atoms_ab]), 'f')

        #coords1 = array([tuple(self.a['C3\''].get_vector()), tuple(self.b['C3\''].get_vector())], 'f')
        #coords2 = array([tuple(other.a['C3\''].get_vector()), tuple(other.b['C3\''].get_vector())], 'f')

        diff = coords1 - coords2 
        l = coords1.shape[0]
        r1 = sqrt(sum(sum(diff*diff))/l)

        return r1

    def reset_coord(self):
        """hack"""
        self.a = self.a_bk.copy()
        self.b = self.b_bk.copy()
        del self.a_bk
        del self.b_bk
        self.atoms_ab = self.get_atoms_ab()
        self.atoms_ba = self.get_atoms_ba()


if '__main__' == __name__:
    parser_pdb = PDB.PDBParser()
    struct = parser_pdb.get_structure('', 'test_data/R_s1t1r7.pdb')
    model = struct[0]
    chain = model['A']

    # 1 43
    a = chain[3]
    b = chain[43]
    print((a,b))


    bp = BasePair(a,b,struct)
    print(('coord:', bp.calc_coord()))
    print((bp.name))
    #dist = bg - bg2

    a = chain[2]
    b = chain[42]
    print((a,b))
    bp2 = BasePair(a,b, struct)
    print(('coord:', bp2.calc_coord()))
    print((bp2.name))

    print(('bp, bp2, distance', bp, bp2, bp2 - bp))

    print(('rmsd:', bp.calc_rmsd_to(bp2)))
    bp.overlap_with(bp2)
    bp.save()
    print(('overlapped rmsd', bp.calc_rmsd_to(bp2)))
    bp.reset_coord()
    print(('reset & rmsd', bp.calc_rmsd_to(bp2)))

    #print bp.get_atoms()



#!/usr/bin/python

from subprocess import Popen, PIPE
from os import remove, path, readlink

PATH = path.abspath(__file__)
if path.islink(PATH):
    PATH = path.dirname(readlink(PATH))
else:
    PATH = path.dirname(path.abspath(__file__))

BINARY_PATH = PATH + '/opt/RNA_PredictionSignificance.app'

def get_p_value(rmsd, length, verbose=False):
    cmd = BINARY_PATH + ' ' + str(length) + ' ' + str(rmsd) 
    out = Popen([cmd], stderr=PIPE, stdout=PIPE, shell=True)
    
    stdout = out.stdout.read()
    outerr = out.stderr.read()

    if verbose: print(stdout)

    pvalues = []
    for l in stdout.split('\n'):
        if l.startswith(' p-value of the prediction: '):
            pvalue = l.replace(' p-value of the prediction: ', '')
            if pvalue.startswith('<'):
                pvalue = float(pvalue[2:])
            else:
                pvalue = float(pvalue)
            pvalues.append(pvalue)
    return pvalues

if __name__ == '__main__':
    print((get_p_value(1,1000)))
    print((get_p_value(1,100)))
    print((get_p_value(1,10)))
    print((get_p_value(1,1)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""

"""
from __future__ import print_function

import argparse
import math

def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    #parser.add_argument('-', "--", help="", default="")

    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    parser.add_argument("kd", help="in M", default="")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    from decimal import Decimal
    kd = float(args.kd)
    print('Kd: %.2e' % Decimal(kd))
    print('pKd', -math.log10(kd))    



#!/usr/bin/env python
import subprocess
import tempfile

from rna_tools.Seq import RNASequence
from rna_tools.rna_tools_config import RFAM_DB_PATH


class RfamSearchError(Exception):
    pass


class RfamSearch():
    """RfamSearch (local).

    Rfam is a collection of multiple sequence alignments and covariance models representing non-coding RNA families. Rfam is available on the web http://rfam.xfam.org/. The website allow the user to search a query sequence against a library of covariance models, and view multiple sequence alignments and family annotation. The database can also be downloaded in flatfile form and searched locally using the INFERNAL package (http://infernal.wustl.edu/). The first release of Rfam (1.0) contains 25 families, which annotate over 50 000 non-coding RNA genes in the taxonomic divisions of the EMBL nucleotide database.

    Infernal ("INFERence of RNA ALignment") is for searching DNA sequence databases for RNA structure and sequence similarities. It is an implementation of a special case of profile stochastic context-free grammars called covariance models (CMs). A CM is like a sequence profile, but it scores a combination of sequence consensus and RNA secondary structure consensus, so in many cases, it is more capable of identifying RNA homologs that conserve their secondary structure more than their primary sequence.

    Infernal `cmscan` is used to search the CM-format Rfam database.

    Setup:

    - download the database from ftp://ftp.ebi.ac.uk/pub/databases/Rfam/CURRENT (file: Rfam.cm.gz, ~30mb)
    - install http://eddylab.org/infernal/
    - set up ``RFAM_DB_PATH`` in the config file of rna-tools.
    - compress Rfam.cm
    
    Example of compressing the database::

         $ cmpress Rfam.cm
         Working...    done.
         Pressed and indexed 3016 CMs and p7 HMM filters (3016 names and 3016 accessions).
         Covariance models and p7 filters pressed into binary file:  Rfam.cm.i1m
         SSI index for binary covariance model file:                 Rfam.cm.i1i
         Optimized p7 filter profiles (MSV part)  pressed into:      Rfam.cm.i1f
         Optimized p7 filter profiles (remainder) pressed into:      Rfam.cm.i1p

    Cite: Nawrocki and S. R. Eddy, Infernal 1.1: 100-fold faster RNA homology searches, Bioinformatics 29:2933-2935 (2013). """  # noqa

    def __init__(self):
        pass

    def cmscan(self, seq, verbose=False):
        """Run cmscan on the seq.

        Usage::

           >>> seq = RNASequence("GGCGCGGCACCGUCCGCGGAACAAACGG")
           >>> rs = RfamSearch()
           >>> hit = rs.cmscan(seq)
           >>> print(hit)  #doctest: +ELLIPSIS
           # cmscan :: search sequence(s) against a CM database...

        :param seq: string
        :returns: result
        :rtype: string """
        #if verbose:
        print('RFAM_DB_PATH', RFAM_DB_PATH)

        # make tmp file
        tf = tempfile.NamedTemporaryFile(delete=False)
        tf.name += '.fa'
        with open(tf.name, 'w') as f:
            f.write('>target\n')
            f.write(seq.seq + '\n')

        # make output file
        of = tempfile.NamedTemporaryFile(delete=False)

        # run cmscan
        cmd = 'cmscan -E 1 ' + RFAM_DB_PATH + ' ' + tf.name + '  > ' + of.name
        print(cmd)
        o = subprocess.Popen(
            cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        # out = o.stdout.read().strip()
        err = o.stderr.read().strip()
        if err:
            raise RfamSearchError(err)
        self.output = open(of.name).read()
        # os.chdir(old_pwd)
        return self.output


# main
if __name__ == '__main__':
    seq = RNASequence("GGCGCGGCACCGUCCGCGGAACAAACGG")
    rs = RfamSearch()
    hit = rs.cmscan(seq)
    print(hit)

    import doctest
    doctest.testmod()




try:
    from pymol import cmd
except ImportError:
    print("PyMOL Python lib is missing")
    # sys.exit(0)

def spl_extract():
 for name in cmd.get_names("all"):
    #if name in ['5zwo', '5gm6', '5lj3', '5mps', '6exn', '5ylz', '5y88', '3jb9', '6icz', '6ff7', '5yzg', '5xjc', '5mq0', '5lj5', '6g90', '6n7r', '6n7p']: # this should be auto
    print(" \ Extracting mode for %s" % name)
    if '5zwo' in name.lower():
        cmd.extract("PRP8_B5zwo", "chain A and 5zwo")
        cmd.extract("BRR2_B5zwo", "chain D and 5zwo")
        cmd.extract("LEA1_B5zwo", "chain o and 5zwo")
        cmd.extract("Msl1_B5zwo", "chain p and 5zwo")
        cmd.extract("SNU114_B5zwo", "chain C and 5zwo")
        cmd.extract("U2_B5zwo", "chain H and 5zwo")
        cmd.extract("U5_B5zwo", "chain B and 5zwo")
        cmd.extract("U6_B5zwo", "chain F and 5zwo")
        cmd.extract("U4_B5zwo", "chain I and 5zwo")
        cmd.extract("Intron_B5zwo", "chain G and 5zwo")
        cmd.extract("PRP4_B5zwo", "chain K and 5zwo")
        cmd.extract("PRP31_B5zwo", "chain L and 5zwo")
        cmd.extract("PRP6_B5zwo", "chain N and 5zwo")
        cmd.extract("PRP3_B5zwo", "chain J and 5zwo")
        cmd.extract("DIB1_B5zwo", "chain E and 5zwo")
        cmd.extract("SNU13_B5zwo", "chain M and 5zwo")
        cmd.extract("LSM8_B5zwo", "chain z and 5zwo")
        cmd.extract("LSM2_B5zwo", "chain q and 5zwo")
        cmd.extract("LSM3_B5zwo", "chain r and 5zwo")
        cmd.extract("LSM6_B5zwo", "chain x and 5zwo")
        cmd.extract("LSM5_B5zwo", "chain t and 5zwo")
        cmd.extract("LSM7_B5zwo", "chain y and 5zwo")
        cmd.extract("LSM4_B5zwo", "chain s and 5zwo")
        cmd.extract("SNU66_B5zwo", "chain O and 5zwo")
        cmd.extract("BUD13_B5zwo", "chain Y and 5zwo")
        cmd.extract("Cus1_B5zwo", "chain 2 and 5zwo")
        cmd.extract("HSH155_B5zwo", "chain 1 and 5zwo")
        cmd.extract("HSH49_B5zwo", "chain 4 and 5zwo")
        cmd.extract("PML1_B5zwo", "chain Z and 5zwo")
        cmd.extract("PRP11_B5zwo", "chain v and 5zwo")
        cmd.extract("RDS3_B5zwo", "chain 5 and 5zwo")
        cmd.extract("RSE1_B5zwo", "chain 3 and 5zwo")
        cmd.extract("SNU17_B5zwo", "chain X and 5zwo")
        cmd.extract("Ysf3_B5zwo", "chain 6 and 5zwo")
        cmd.extract("SMB1_1_B5zwo", "chain a and 5zwo")
        cmd.extract("SMB1_2_B5zwo", "chain P and 5zwo")
        cmd.extract("SMB1_3_B5zwo", "chain h and 5zwo")
        cmd.extract("SME1_1_B5zwo", "chain e and 5zwo")
        cmd.extract("SME1_2_B5zwo", "chain T and 5zwo")
        cmd.extract("SME1_3_B5zwo", "chain i and 5zwo")
        cmd.extract("SMX3_1_B5zwo", "chain f and 5zwo")
        cmd.extract("SMX3_2_B5zwo", "chain U and 5zwo")
        cmd.extract("SMX3_3_B5zwo", "chain j and 5zwo")
        cmd.extract("SMX2_1_B5zwo", "chain g and 5zwo")
        cmd.extract("SMX2_2_B5zwo", "chain V and 5zwo")
        cmd.extract("SMX2_3_B5zwo", "chain k and 5zwo")
        cmd.extract("SMD3_1_B5zwo", "chain d and 5zwo")
        cmd.extract("SMD3_2_B5zwo", "chain S and 5zwo")
        cmd.extract("SMD3_3_B5zwo", "chain l and 5zwo")
        cmd.extract("SMD1_1_B5zwo", "chain b and 5zwo")
        cmd.extract("SMD1_2_B5zwo", "chain Q and 5zwo")
        cmd.extract("SMD1_3_B5zwo", "chain m and 5zwo")
        cmd.extract("SMD2_1_B5zwo", "chain c and 5zwo")
        cmd.extract("SMD2_2_B5zwo", "chain R and 5zwo")
        cmd.extract("SMD2_3_B5zwo", "chain n and 5zwo")
        cmd.extract("PRP9_B5zwo", "chain u and 5zwo")
        cmd.extract("PRP21_B5zwo", "chain w and 5zwo")
        cmd.extract("SNU23_B5zwo", "chain W and 5zwo")
        cmd.extract("PRP38_B5zwo", "chain 0 and 5zwo")
        cmd.extract("SPP381_B5zwo", "chain 9 and 5zwo")
        cmd.set_name("5zwo", "unknown_other_B5zwo")
        cmd.group("B5zwo", "*_B5zwo")
        cmd.do("order *, yes")
    if '5gm6' in name.lower():
        cmd.extract("PRP8_Ba5gm6", "chain A and 5gm6")
        cmd.extract("BRR2_Ba5gm6", "chain B and 5gm6")
        cmd.extract("BUD31_Ba5gm6", "chain T and 5gm6")
        cmd.extract("CEF1_Ba5gm6", "chain c and 5gm6")
        cmd.extract("CWC15_Ba5gm6", "chain S and 5gm6")
        cmd.extract("CWC2_hRBM22_Ba5gm6", "chain R and 5gm6")
        cmd.extract("CWC21_Ba5gm6", "chain X and 5gm6")
        cmd.extract("CWC22_Ba5gm6", "chain Z and 5gm6")
        cmd.extract("PRP45_Ba5gm6", "chain P and 5gm6")
        cmd.extract("CDC40_Ba5gm6", "chain n and 5gm6")
        cmd.extract("PRP19_Ba5gm6", "chain f and 5gm6")
        cmd.extract("PRP46_Ba5gm6", "chain O and 5gm6")
        cmd.extract("SLT11/ECM2_Ba5gm6", "chain Q and 5gm6")
        cmd.extract("SNT309_Ba5gm6", "chain t and 5gm6")
        cmd.extract("SNU114_Ba5gm6", "chain C and 5gm6")
        cmd.extract("SYF2_Ba5gm6", "chain f and 5gm6")
        cmd.extract("SYF1_Ba5gm6", "chain v and 5gm6")
        cmd.extract("U2_Ba5gm6", "chain 2 and 5gm6")
        cmd.extract("U5_Ba5gm6", "chain 5 and 5gm6")
        cmd.extract("U6_Ba5gm6", "chain 6 and 5gm6")
        cmd.extract("Intron_Ba5gm6", "chain M and 5gm6")
        cmd.extract("Exon_Ba5gm6", "chain N and 5gm6")
        cmd.extract("BUD13_Ba5gm6", "chain W and 5gm6")
        cmd.extract("CLF2_Ba5gm6", "chain d and 5gm6")
        cmd.extract("Cus1_Ba5gm6", "chain H and 5gm6")
        cmd.extract("CWC24_Ba5gm6", "chain a and 5gm6")
        cmd.extract("CWC27_Ba5gm6", "chain b and 5gm6")
        cmd.extract("HSH155_Ba5gm6", "chain G and 5gm6")
        cmd.extract("HSH49_Ba5gm6", "chain e and 5gm6")
        cmd.extract("PML1_Ba5gm6", "chain U and 5gm6")
        cmd.extract("PRP11_Ba5gm6", "chain I and 5gm6")
        cmd.extract("PRP2_Ba5gm6", "chain Y and 5gm6")
        cmd.extract("RDS3_Ba5gm6", "chain J and 5gm6")
        cmd.extract("RSE1_Ba5gm6", "chain F and 5gm6")
        cmd.extract("SNU17_Ba5gm6", "chain V and 5gm6")
        cmd.extract("Ysf3_Ba5gm6", "chain K and 5gm6")
        cmd.set_name("5gm6", "unknown_other_Ba5gm6")
        cmd.group("Ba5gm6", "*_Ba5gm6")
        cmd.do("order *, yes")
    if '5lj3' in name.lower():
        cmd.extract("PRP8_C5lj3", "chain A and 5lj3")
        cmd.extract("BUD31_C5lj3", "chain L and 5lj3")
        cmd.extract("CEF1_C5lj3", "chain O and 5lj3")
        cmd.extract("CLF1_C5lj3", "chain S and 5lj3")
        cmd.extract("CWC15_C5lj3", "chain P and 5lj3")
        cmd.extract("CWC16/YJU2_C5lj3", "chain D and 5lj3")
        cmd.extract("CWC2_hRBM22_C5lj3", "chain M and 5lj3")
        cmd.extract("CWC21_C5lj3", "chain R and 5lj3")
        cmd.extract("CWC22_C5lj3", "chain H and 5lj3")
        cmd.extract("CWC25_C5lj3", "chain F and 5lj3")
        cmd.extract("ISY1_C5lj3", "chain G and 5lj3")
        cmd.extract("LEA1_C5lj3", "chain W and 5lj3")
        cmd.extract("Msl1_C5lj3", "chain Y and 5lj3")
        cmd.extract("PRP45_C5lj3", "chain K and 5lj3")
        cmd.extract("PRP46_C5lj3", "chain J and 5lj3")
        cmd.extract("SLT11/ECM2_C5lj3", "chain N and 5lj3")
        cmd.extract("SNU114_C5lj3", "chain C and 5lj3")
        cmd.extract("SYF1_C5lj3", "chain T and 5lj3")
        cmd.extract("U2_C5lj3", "chain Z and 5lj3")
        cmd.extract("U5_C5lj3", "chain U and 5lj3")
        cmd.extract("U6_C5lj3", "chain V and 5lj3")
        cmd.extract("Intron_C5lj3", "chain I and 5lj3")
        cmd.extract("Exon_C5lj3", "chain E and 5lj3")
        cmd.extract("SMB1_1_C5lj3", "chain b and 5lj3")
        cmd.extract("SMB1_2_C5lj3", "chain k and 5lj3")
        cmd.extract("SME1_1_C5lj3", "chain e and 5lj3")
        cmd.extract("SME1_2_C5lj3", "chain p and 5lj3")
        cmd.extract("SMX3_1_C5lj3", "chain f and 5lj3")
        cmd.extract("SMX3_2_C5lj3", "chain q and 5lj3")
        cmd.extract("SMX2_1_C5lj3", "chain g and 5lj3")
        cmd.extract("SMX2_2_C5lj3", "chain r and 5lj3")
        cmd.extract("SMD3_1_C5lj3", "chain d and 5lj3")
        cmd.extract("SMD3_2_C5lj3", "chain n and 5lj3")
        cmd.extract("SMD1_1_C5lj3", "chain h and 5lj3")
        cmd.extract("SMD1_2_C5lj3", "chain l and 5lj3")
        cmd.extract("SMD2_1_C5lj3", "chain j and 5lj3")
        cmd.extract("SMD2_2_C5lj3", "chain m and 5lj3")
        cmd.set_name("5lj3", "unknown_other_C5lj3")
        cmd.group("C5lj3", "*_C5lj3")
        cmd.do("order *, yes")
    if '5mps' in name.lower():
        cmd.extract("PRP8_Cs5mps", "chain A and 5mps")
        cmd.extract("BUD31_Cs5mps", "chain L and 5mps")
        cmd.extract("CEF1_Cs5mps", "chain O and 5mps")
        cmd.extract("CLF1_Cs5mps", "chain S and 5mps")
        cmd.extract("CWC15_Cs5mps", "chain P and 5mps")
        cmd.extract("CWC2_hRBM22_Cs5mps", "chain M and 5mps")
        cmd.extract("CWC21_Cs5mps", "chain R and 5mps")
        cmd.extract("CWC22_Cs5mps", "chain H and 5mps")
        cmd.extract("PRP45_Cs5mps", "chain K and 5mps")
        cmd.extract("CDC40_Cs5mps", "chain o and 5mps")
        cmd.extract("PRP46_Cs5mps", "chain J and 5mps")
        cmd.extract("SLT11/ECM2_Cs5mps", "chain N and 5mps")
        cmd.extract("SNU114_Cs5mps", "chain C and 5mps")
        cmd.extract("SYF2_Cs5mps", "chain y and 5mps")
        cmd.extract("SYF1_Cs5mps", "chain T and 5mps")
        cmd.extract("U2_Cs5mps", "chain 2 and 5mps")
        cmd.extract("U5_Cs5mps", "chain 5 and 5mps")
        cmd.extract("U6_Cs5mps", "chain 6 and 5mps")
        cmd.extract("5EXON_Cs5mps", "chain E and 5mps")
        cmd.extract("Intron_Cs5mps", "chain I and 5mps")
        cmd.extract("Exon_Cs5mps", "chain E and 5mps")
        cmd.extract("SMB1_Cs5mps", "chain b and 5mps")
        cmd.extract("SME1_Cs5mps", "chain e and 5mps")
        cmd.extract("SMX3_Cs5mps", "chain f and 5mps")
        cmd.extract("SMX2_Cs5mps", "chain g and 5mps")
        cmd.extract("SMD3_Cs5mps", "chain d and 5mps")
        cmd.extract("SMD1_Cs5mps", "chain h and 5mps")
        cmd.extract("SMD2_Cs5mps", "chain j and 5mps")
        cmd.extract("PRP18_Cs5mps", "chain a and 5mps")
        cmd.extract("SLU7_Cs5mps", "chain c and 5mps")
        cmd.set_name("5mps", "unknown_other_Cs5mps")
        cmd.group("Cs5mps", "*_Cs5mps")
        cmd.do("order *, yes")
    if '6exn' in name.lower():
        cmd.extract("PRP8_P6exn", "chain A and 6exn")
        cmd.extract("BUD31_P6exn", "chain L and 6exn")
        cmd.extract("CEF1_P6exn", "chain O and 6exn")
        cmd.extract("CLF1_P6exn", "chain S and 6exn")
        cmd.extract("CWC15_P6exn", "chain P and 6exn")
        cmd.extract("CWC16/YJU2_P6exn", "chain D and 6exn")
        cmd.extract("CWC2_hRBM22_P6exn", "chain M and 6exn")
        cmd.extract("CWC21_P6exn", "chain R and 6exn")
        cmd.extract("CWC22_P6exn", "chain H and 6exn")
        cmd.extract("LEA1_P6exn", "chain W and 6exn")
        cmd.extract("Msl1_P6exn", "chain Y and 6exn")
        cmd.extract("PRP45_P6exn", "chain K and 6exn")
        cmd.extract("CDC40_P6exn", "chain o and 6exn")
        cmd.extract("PRP19_1_P6exn", "chain t and 6exn")
        cmd.extract("PRP19_2_P6exn", "chain u and 6exn")
        cmd.extract("PRP19_3_P6exn", "chain v and 6exn")
        cmd.extract("PRP19_4_P6exn", "chain w and 6exn")
        cmd.extract("PRP46_P6exn", "chain J and 6exn")
        cmd.extract("SLT11/ECM2_P6exn", "chain N and 6exn")
        cmd.extract("SNU114_P6exn", "chain C and 6exn")
        cmd.extract("SYF1_P6exn", "chain T and 6exn")
        cmd.extract("U2_P6exn", "chain 2 and 6exn")
        cmd.extract("U5_P6exn", "chain 5 and 6exn")
        cmd.extract("U6_P6exn", "chain 6 and 6exn")
        cmd.extract("Intron_P6exn", "chain I and 6exn")
        cmd.extract("Exon_P6exn", "chain E and 6exn")
        cmd.extract("SMB1_1_P6exn", "chain b and 6exn")
        cmd.extract("SMB1_2_P6exn", "chain k and 6exn")
        cmd.extract("SME1_1_P6exn", "chain e and 6exn")
        cmd.extract("SME1_2_P6exn", "chain p and 6exn")
        cmd.extract("SMX3_1_P6exn", "chain f and 6exn")
        cmd.extract("SMX3_2_P6exn", "chain q and 6exn")
        cmd.extract("SMX2_1_P6exn", "chain g and 6exn")
        cmd.extract("SMX2_2_P6exn", "chain r and 6exn")
        cmd.extract("SMD3_1_P6exn", "chain d and 6exn")
        cmd.extract("SMD3_2_P6exn", "chain n and 6exn")
        cmd.extract("SMD1_1_P6exn", "chain h and 6exn")
        cmd.extract("SMD1_2_P6exn", "chain l and 6exn")
        cmd.extract("SMD2_1_P6exn", "chain j and 6exn")
        cmd.extract("SMD2_2_P6exn", "chain m and 6exn")
        cmd.extract("PRP22_P6exn", "chain V and 6exn")
        cmd.extract("PRP18_P6exn", "chain a and 6exn")
        cmd.extract("SLU7_P6exn", "chain c and 6exn")
        cmd.extract("unassigned_P6exn", "chain X and 6exn")
        cmd.set_name("6exn", "unknown_other_P6exn")
        cmd.group("P6exn", "*_P6exn")
        cmd.do("order *, yes")
    if '5ylz' in name.lower():
        cmd.extract("PRP8_P5ylz", "chain A and 5ylz")
        cmd.extract("BUD31_P5ylz", "chain L and 5ylz")
        cmd.extract("CEF1_P5ylz", "chain J and 5ylz")
        cmd.extract("CLF1_P5ylz", "chain I and 5ylz")
        cmd.extract("CWC15_P5ylz", "chain P and 5ylz")
        cmd.extract("CWC2_hRBM22_P5ylz", "chain N and 5ylz")
        cmd.extract("CWC21_P5ylz", "chain R and 5ylz")
        cmd.extract("CWC22_P5ylz", "chain S and 5ylz")
        cmd.extract("LEA1_P5ylz", "chain o and 5ylz")
        cmd.extract("Msl1_P5ylz", "chain p and 5ylz")
        cmd.extract("PRP45_P5ylz", "chain Q and 5ylz")
        cmd.extract("CDC40_P5ylz", "chain T and 5ylz")
        cmd.extract("PRP19_1_P5ylz", "chain q and 5ylz")
        cmd.extract("PRP19_2_P5ylz", "chain r and 5ylz")
        cmd.extract("PRP19_3_P5ylz", "chain s and 5ylz")
        cmd.extract("PRP19_4_P5ylz", "chain t and 5ylz")
        cmd.extract("PRP46_P5ylz", "chain O and 5ylz")
        cmd.extract("SLT11/ECM2_P5ylz", "chain M and 5ylz")
        cmd.extract("SNT309_P5ylz", "chain G and 5ylz")
        cmd.extract("SNU114_P5ylz", "chain C and 5ylz")
        cmd.extract("SYF2_P5ylz", "chain K and 5ylz")
        cmd.extract("SYF1_P5ylz", "chain H and 5ylz")
        cmd.extract("U2_P5ylz", "chain F and 5ylz")
        cmd.extract("U5_P5ylz", "chain B and 5ylz")
        cmd.extract("U6_P5ylz", "chain D and 5ylz")
        cmd.extract("Intron_P5ylz", "chain E and 5ylz")
        cmd.extract("SMB1_1_P5ylz", "chain a and 5ylz")
        cmd.extract("SMB1_2_P5ylz", "chain h and 5ylz")
        cmd.extract("SME1_1_P5ylz", "chain b and 5ylz")
        cmd.extract("SME1_2_P5ylz", "chain i and 5ylz")
        cmd.extract("SMX3_1_P5ylz", "chain c and 5ylz")
        cmd.extract("SMX3_2_P5ylz", "chain j and 5ylz")
        cmd.extract("SMX2_1_P5ylz", "chain d and 5ylz")
        cmd.extract("SMX2_2_P5ylz", "chain k and 5ylz")
        cmd.extract("SMD3_1_P5ylz", "chain e and 5ylz")
        cmd.extract("SMD3_2_P5ylz", "chain l and 5ylz")
        cmd.extract("SMD1_1_P5ylz", "chain f and 5ylz")
        cmd.extract("SMD1_2_P5ylz", "chain m and 5ylz")
        cmd.extract("SMD2_1_P5ylz", "chain g and 5ylz")
        cmd.extract("SMD2_2_P5ylz", "chain n and 5ylz")
        cmd.extract("PRP22_P5ylz", "chain W and 5ylz")
        cmd.extract("PRP18_P5ylz", "chain U and 5ylz")
        cmd.extract("SLU7_P5ylz", "chain V and 5ylz")
        cmd.set_name("5ylz", "unknown_other_P5ylz")
        cmd.group("P5ylz", "*_P5ylz")
        cmd.do("order *, yes")
    if '5y88' in name.lower():
        cmd.extract("PRP8_I5y88", "chain A and 5y88")
        cmd.extract("BUD31_I5y88", "chain L and 5y88")
        cmd.extract("CLF1_I5y88", "chain I and 5y88")
        cmd.extract("CWC15_I5y88", "chain P and 5y88")
        cmd.extract("CWC16/YJU2_I5y88", "chain R and 5y88")
        cmd.extract("CWC2_hRBM22_I5y88", "chain N and 5y88")
        cmd.extract("CWC25_I5y88", "chain G and 5y88")
        cmd.extract("Intron_2_I5y88", "chain E and 5y88")
        cmd.extract("LEA1_I5y88", "chain o and 5y88")
        cmd.extract("Msl1_I5y88", "chain p and 5y88")
        cmd.extract("PRP45_I5y88", "chain Q and 5y88")
        cmd.extract("CDC40_I5y88", "chain S and 5y88")
        cmd.extract("PRP19_1_I5y88", "chain q and 5y88")
        cmd.extract("PRP19_2_I5y88", "chain r and 5y88")
        cmd.extract("PRP19_3_I5y88", "chain s and 5y88")
        cmd.extract("PRP19_4_I5y88", "chain t and 5y88")
        cmd.extract("PRP46_I5y88", "chain O and 5y88")
        cmd.extract("SLT11/ECM2_I5y88", "chain M and 5y88")
        cmd.extract("SNT309_I5y88", "chain G and 5y88")
        cmd.extract("SNU114_I5y88", "chain C and 5y88")
        cmd.extract("SYF2_I5y88", "chain K and 5y88")
        cmd.extract("SYF1_I5y88", "chain H and 5y88")
        cmd.extract("U2_I5y88", "chain F and 5y88")
        cmd.extract("U5_I5y88", "chain B and 5y88")
        cmd.extract("U6_I5y88", "chain D and 5y88")
        cmd.extract("Intron_I5y88", "chain x and 5y88")
        cmd.extract("RNA_I5y88", "chain x and 5y88")
        cmd.extract("cwc23_I5y88", "chain T and 5y88")
        cmd.extract("SPP382_I5y88", "chain U and 5y88")
        cmd.extract("NTR2_I5y88", "chain V and 5y88")
        cmd.extract("PRP43_I5y88", "chain W and 5y88")
        cmd.extract("SMB1_1_I5y88", "chain a and 5y88")
        cmd.extract("SMB1_2_I5y88", "chain h and 5y88")
        cmd.extract("SME1_1_I5y88", "chain b and 5y88")
        cmd.extract("SME1_2_I5y88", "chain i and 5y88")
        cmd.extract("SMX3_1_I5y88", "chain c and 5y88")
        cmd.extract("SMX3_2_I5y88", "chain j and 5y88")
        cmd.extract("SMX2_1_I5y88", "chain d and 5y88")
        cmd.extract("SMX2_2_I5y88", "chain k and 5y88")
        cmd.extract("SMD3_1_I5y88", "chain e and 5y88")
        cmd.extract("SMD3_2_I5y88", "chain l and 5y88")
        cmd.extract("SMD1_1_I5y88", "chain f and 5y88")
        cmd.extract("SMD1_2_I5y88", "chain m and 5y88")
        cmd.extract("SMD2_1_I5y88", "chain g and 5y88")
        cmd.extract("SMD2_2_I5y88", "chain n and 5y88")
        cmd.set_name("5y88", "unknown_other_I5y88")
        cmd.group("I5y88", "*_I5y88")
        cmd.do("order *, yes")
    if '3jb9' in name.lower():
        cmd.extract("U2_3jb9", "chain P and 3jb9")
        cmd.extract("U5_3jb9", "chain C and 3jb9")
        cmd.extract("U6_3jb9", "chain N and 3jb9")
        cmd.extract("Intron_1_3jb9", "chain O and 3jb9")
        cmd.extract("Intron_2_3jb9", "chain Q and 3jb9")
        cmd.extract("Spp42_yPrp8_3jb9", "chain A and 3jb9")
        cmd.extract("CWF15_yCWC15_3jb9", "chain h and 3jb9")
        cmd.set_name("3jb9", "unknown_other_3jb9")
        cmd.group("3jb9", "*_3jb9")
        cmd.do("order *, yes")
    if '6icz' in name.lower():
        cmd.extract("CWC15_hP_6icz", "chain P and 6icz")
        cmd.extract("U2_hP_6icz", "chain H and 6icz")
        cmd.extract("U5_hP_6icz", "chain B and 6icz")
        cmd.extract("U6_hP_6icz", "chain F and 6icz")
        cmd.extract("Intron_hP_6icz", "chain G and 6icz")
        cmd.extract("cwc23_hP_6icz", "chain 6ICZ and 6icz")
        cmd.set_name("6icz", "unknown_other_hP_6icz")
        cmd.group("hP6icz", "*_hP_6icz")
        cmd.do("order *, yes")
    if '6ff7' in name.lower():
        cmd.extract("CWC15_hBa_6ff7", "chain R and 6ff7")
        cmd.extract("CWC2_hRBM22_hBa_6ff7", "chain P and 6ff7")
        cmd.extract("U2_hBa_6ff7", "chain 2 and 6ff7")
        cmd.extract("U5_hBa_6ff7", "chain 5 and 6ff7")
        cmd.extract("U6_hBa_6ff7", "chain 6 and 6ff7")
        cmd.extract("Intron_hBa_6ff7", "chain Z and 6ff7")
        cmd.set_name("6ff7", "unknown_other_hBa_6ff7")
        cmd.group("hBa6ff7", "*_hBa_6ff7")
        cmd.do("order *, yes")
    if '5yzg' in name.lower():
        cmd.extract("CWC15_hC_5yzg", "chain P and 5yzg")
        cmd.extract("CWC2_hRBM22_hC_5yzg", "chain O and 5yzg")
        cmd.extract("CWC25_hC_5yzg", "chain X and 5yzg")
        cmd.extract("PRP16_hDHX38_hC_5yzg", "chain Z and 5yzg")
        cmd.extract("U2_hC_5yzg", "chain H and 5yzg")
        cmd.extract("U5_hC_5yzg", "chain B and 5yzg")
        cmd.extract("U6_hC_5yzg", "chain F and 5yzg")
        cmd.extract("Intron_hC_5yzg", "chain G and 5yzg")
        cmd.set_name("5yzg", "unknown_other_hC_5yzg")
        cmd.group("hC5yzg", "*_hC_5yzg")
        cmd.do("order *, yes")
    if '5xjc' in name.lower():
        cmd.extract("CWC15_hX_5xjc", "chain P and 5xjc")
        cmd.extract("CWC25_hX_5xjc", "chain X and 5xjc")
        cmd.extract("U2_hX_5xjc", "chain H and 5xjc")
        cmd.extract("U5_hX_5xjc", "chain B and 5xjc")
        cmd.extract("U6_hX_5xjc", "chain F and 5xjc")
        cmd.extract("Intron_hX_5xjc", "chain G and 5xjc")
        cmd.extract("PRKRIP1_hX_5xjc", "chain X and 5xjc")
        cmd.set_name("5xjc", "unknown_other_hX_5xjc")
        cmd.group("hX5xjc", "*_hX_5xjc")
        cmd.do("order *, yes")
    if '5mq0' in name.lower():
        cmd.extract("CWC15_yCs_5mq0", "chain P and 5mq0")
        cmd.extract("CWC2_hRBM22_yCs_5mq0", "chain M and 5mq0")
        cmd.extract("U2_yCs_5mq0", "chain 2 and 5mq0")
        cmd.extract("U5_yCs_5mq0", "chain 5 and 5mq0")
        cmd.extract("U6_yCs_5mq0", "chain 6 and 5mq0")
        cmd.extract("5EXON_yCs_5mq0", "chain E and 5mq0")
        cmd.extract("Intron_yCs_5mq0", "chain I and 5mq0")
        cmd.extract("exon-3_yCs_5mq0", "chain 3 and 5mq0")
        cmd.set_name("5mq0", "unknown_other_yCs_5mq0")
        cmd.group("yCs5mq0", "*_yCs_5mq0")
        cmd.do("order *, yes")
    if '5lj5' in name.lower():
        cmd.extract("PRP8_yC_5lj5", "chain A and 5lj5")
        cmd.extract("BRR2_yC_5lj5", "chain B and 5lj5")
        cmd.extract("BUD31_yC_5lj5", "chain L and 5lj5")
        cmd.extract("CEF1_yC_5lj5", "chain O and 5lj5")
        cmd.extract("CLF1_yC_5lj5", "chain S and 5lj5")
        cmd.extract("CWC15_yC_5lj5", "chain P and 5lj5")
        cmd.extract("CWC16/YJU2_yC_5lj5", "chain D and 5lj5")
        cmd.extract("CWC2_hRBM22_yC_5lj5", "chain M and 5lj5")
        cmd.extract("CWC21_yC_5lj5", "chain R and 5lj5")
        cmd.extract("CWC22_yC_5lj5", "chain H and 5lj5")
        cmd.extract("CWC25_yC_5lj5", "chain F and 5lj5")
        cmd.extract("ISY1_yC_5lj5", "chain G and 5lj5")
        cmd.extract("Msl1_yC_5lj5", "chain Y and 5lj5")
        cmd.extract("PRP45_yC_5lj5", "chain K and 5lj5")
        cmd.extract("PRP16_hDHX38_yC_5lj5", "chain Q and 5lj5")
        cmd.extract("PRP19_1_yC_5lj5", "chain t and 5lj5")
        cmd.extract("PRP19_2_yC_5lj5", "chain u and 5lj5")
        cmd.extract("PRP19_3_yC_5lj5", "chain v and 5lj5")
        cmd.extract("PRP19_4_yC_5lj5", "chain w and 5lj5")
        cmd.extract("PRP46_yC_5lj5", "chain J and 5lj5")
        cmd.extract("SLT11/ECM2_yC_5lj5", "chain N and 5lj5")
        cmd.extract("SNT309_yC_5lj5", "chain s and 5lj5")
        cmd.extract("SNU114_yC_5lj5", "chain C and 5lj5")
        cmd.extract("SYF1_yC_5lj5", "chain T and 5lj5")
        cmd.extract("U2_yC_5lj5", "chain Z and 5lj5")
        cmd.extract("U5_yC_5lj5", "chain U and 5lj5")
        cmd.extract("U6_yC_5lj5", "chain V and 5lj5")
        cmd.extract("5EXON_yC_5lj5", "chain E and 5lj5")
        cmd.extract("Intron_yC_5lj5", "chain I and 5lj5")
        cmd.extract("SMB1_1_yC_5lj5", "chain b and 5lj5")
        cmd.extract("SMB1_2_yC_5lj5", "chain k and 5lj5")
        cmd.extract("SME1_1_yC_5lj5", "chain e and 5lj5")
        cmd.extract("SME1_2_yC_5lj5", "chain p and 5lj5")
        cmd.extract("SMX3_1_yC_5lj5", "chain f and 5lj5")
        cmd.extract("SMX3_2_yC_5lj5", "chain q and 5lj5")
        cmd.extract("SMX2_1_yC_5lj5", "chain g and 5lj5")
        cmd.extract("SMX2_2_yC_5lj5", "chain r and 5lj5")
        cmd.extract("SMD3_1_yC_5lj5", "chain d and 5lj5")
        cmd.extract("SMD3_2_yC_5lj5", "chain n and 5lj5")
        cmd.extract("SMD1_1_yC_5lj5", "chain h and 5lj5")
        cmd.extract("SMD1_2_yC_5lj5", "chain l and 5lj5")
        cmd.extract("SMD2_1_yC_5lj5", "chain j and 5lj5")
        cmd.extract("SMD2_2_yC_5lj5", "chain m and 5lj5")
        cmd.extract("unassigned_yC_5lj5", "chain x and 5lj5")
        cmd.set_name("5lj5", "unknown_other_yC_5lj5")
        cmd.group("yC5lj5", "*_yC_5lj5")
        cmd.do("order *, yes")
    if '6g90' in name.lower():
        cmd.extract("LEA1_yPre_6g90", "chain W and 6g90")
        cmd.extract("Msl1_yPre_6g90", "chain Y and 6g90")
        cmd.extract("Ysf3_yPre_6g90", "chain Z and 6g90")
        cmd.extract("SMB1_1_yPre_6g90", "chain b and 6g90")
        cmd.extract("SMB1_2_yPre_6g90", "chain s and 6g90")
        cmd.extract("SME1_1_yPre_6g90", "chain e and 6g90")
        cmd.extract("SME1_2_yPre_6g90", "chain w and 6g90")
        cmd.extract("SMX3_1_yPre_6g90", "chain f and 6g90")
        cmd.extract("SMX3_2_yPre_6g90", "chain x and 6g90")
        cmd.extract("SMX2_1_yPre_6g90", "chain g and 6g90")
        cmd.extract("SMX2_2_yPre_6g90", "chain y and 6g90")
        cmd.extract("SMD3_1_yPre_6g90", "chain d and 6g90")
        cmd.extract("SMD3_2_yPre_6g90", "chain v and 6g90")
        cmd.extract("SMD1_1_yPre_6g90", "chain h and 6g90")
        cmd.extract("SMD1_2_yPre_6g90", "chain t and 6g90")
        cmd.extract("SMD2_1_yPre_6g90", "chain i and 6g90")
        cmd.extract("SMD2_2_yPre_6g90", "chain u and 6g90")
        cmd.extract("unassigned_yPre_6g90", "chain X and 6g90")
        cmd.extract("MUD1_yPre_6g90", "chain A and 6g90")
        cmd.extract("SNP1_yPre_6g90", "chain B and 6g90")
        cmd.extract("YHC1_yPre_6g90", "chain C and 6g90")
        cmd.extract("PRP39_yPre_6g90", "chain D and 6g90")
        cmd.extract("PRP42_yPre_6g90", "chain E and 6g90")
        cmd.extract("NAM8_yPre_6g90", "chain F and 6g90")
        cmd.extract("SNU56_yPre_6g90", "chain G and 6g90")
        cmd.extract("LUC7_yPre_6g90", "chain H and 6g90")
        cmd.extract("SNU71_yPre_6g90", "chain J and 6g90")
        cmd.extract("HSH155_yPre_6g90", "chain O and 6g90")
        cmd.extract("RSE1_yPre_6g90", "chain P and 6g90")
        cmd.extract("CUS1_yPre_6g90", "chain Q and 6g90")
        cmd.extract("HSH49_yPre_6g90", "chain R and 6g90")
        cmd.extract("RDS3_yPre_6g90", "chain S and 6g90")
        cmd.extract("PRP9_yPre_6g90", "chain T and 6g90")
        cmd.extract("PRP11_yPre_6g90", "chain U and 6g90")
        cmd.extract("PRP21_yPre_6g90", "chain V and 6g90")
        cmd.set_name("6g90", "unknown_other_yPre_6g90")
        cmd.group("yPre6g90", "*_yPre_6g90")
        cmd.do("order *, yes")
    if '6n7r' in name.lower():
        cmd.extract("Intron_yE_6n7r", "chain r and 6n7r")
        cmd.extract("SMB1_yE_6n7r", "chain K and 6n7r")
        cmd.extract("SME1_yE_6n7r", "chain O and 6n7r")
        cmd.extract("SMX3_yE_6n7r", "chain P and 6n7r")
        cmd.extract("SMX2_yE_6n7r", "chain Q and 6n7r")
        cmd.extract("SMD3_yE_6n7r", "chain N and 6n7r")
        cmd.extract("SMD1_yE_6n7r", "chain L and 6n7r")
        cmd.extract("SMD2_yE_6n7r", "chain M and 6n7r")
        cmd.extract("MUD1_yE_6n7r", "chain C and 6n7r")
        cmd.extract("SNP1_yE_6n7r", "chain A and 6n7r")
        cmd.extract("YHC1_yE_6n7r", "chain B and 6n7r")
        cmd.extract("PRP39_yE_6n7r", "chain E and 6n7r")
        cmd.extract("PRP42_yE_6n7r", "chain D and 6n7r")
        cmd.extract("NAM8_yE_6n7r", "chain F and 6n7r")
        cmd.extract("SNU56_yE_6n7r", "chain G and 6n7r")
        cmd.extract("LUC7_yE_6n7r", "chain I and 6n7r")
        cmd.extract("SNU71_yE_6n7r", "chain H and 6n7r")
        cmd.extract("U1_yE_6n7r", "chain R and 6n7r")
        cmd.set_name("6n7r", "unknown_other_yE_6n7r")
        cmd.group("yE6n7r", "*_yE_6n7r")
        cmd.do("order *, yes")
    if '6n7p' in name.lower():
        cmd.extract("Intron_yE_6n7p", "chain r and 6n7p")
        cmd.extract("SMB1_yE_6n7p", "chain K and 6n7p")
        cmd.extract("SME1_yE_6n7p", "chain O and 6n7p")
        cmd.extract("SMX3_yE_6n7p", "chain P and 6n7p")
        cmd.extract("SMX2_yE_6n7p", "chain Q and 6n7p")
        cmd.extract("SMD3_yE_6n7p", "chain N and 6n7p")
        cmd.extract("SMD1_yE_6n7p", "chain L and 6n7p")
        cmd.extract("SMD2_yE_6n7p", "chain M and 6n7p")
        cmd.extract("MUD1_yE_6n7p", "chain C and 6n7p")
        cmd.extract("SNP1_yE_6n7p", "chain A and 6n7p")
        cmd.extract("YHC1_yE_6n7p", "chain B and 6n7p")
        cmd.extract("PRP39_yE_6n7p", "chain E and 6n7p")
        cmd.extract("PRP42_yE_6n7p", "chain D and 6n7p")
        cmd.extract("NAM8_yE_6n7p", "chain F and 6n7p")
        cmd.extract("SNU56_yE_6n7p", "chain G and 6n7p")
        cmd.extract("LUC7_yE_6n7p", "chain I and 6n7p")
        cmd.extract("SNU71_yE_6n7p", "chain H and 6n7p")
        cmd.extract("U1_yE_6n7p", "chain R and 6n7p")
        cmd.extract("PRP40_yE_6n7p", "chain J and 6n7p")
        cmd.extract("STO1_yE_6n7p", "chain X and 6n7p")
        cmd.extract("CBC2_yE_6n7p", "chain Y and 6n7p")
        cmd.set_name("6n7p", "unknown_other_yE_6n7p")
        cmd.group("yE6n7p", "*_yE_6n7p")
        cmd.do("order *, yes")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_pairs2SimRNArestrs.py - convert pairs to SimRNA restraints

Example::

    $ rna_pairs2SimRNArestrs.py rp06_pairs_delta.txt -v
    # of pairs: 42
    SLOPE A/2/MB A/172/MB 0 6 1
    SLOPE A/2/MB A/172/MB 0 7 -1
    SLOPE A/3/MB A/169/MB 0 6 1
    SLOPE A/3/MB A/169/MB 0 7 -1
    SLOPE A/12/MB A/32/MB 0 6 1

"""
from __future__ import print_function
import argparse


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument('pairs', help="a file with [[2, 172], [3, 169], [12, 32], [13, 31]]")
    parser.add_argument("--offset", help="can be -10", default=0, type=int)
    parser.add_argument("--weight", default=3, type=float, help="weight")
    parser.add_argument("--dist", default=7, help="distances, for MOHCA use 25", type=int)
    parser.add_argument("--well", help="well instead of slope", action="store_true", default=False)
    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    return parser


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    pairs = eval(open(args.pairs).read().strip())
    if args.verbose:
        print('# of pairs:', len(pairs))

    for pair in pairs:

        a, b = pair
        a += args.offset
        b += args.offset

        a = str(a)
        b = str(b)

        if args.well:
            print('WELL A/' + a + '/MB A/' + b + '/MB 0 ' + str(args.dist) + ' ' + str(args.weight))
        else:
            print('SLOPE A/' + a + '/MB A/' + b + '/MB 0 ' + str(args.dist - 1) + ' ' + str(args.weight))
            print('SLOPE A/' + a + '/MB A/' + b + '/MB 0 ' + str(args.dist) + ' -' + str(args.weight))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""rna_rosetta_cluster.py - cluster silent Rosetta files

A wrapper to ROSETTA tools for RNA modeling

Based on C. Y. Cheng, F. C. Chou, and R. Das, Modeling complex RNA tertiary folds with Rosetta, 1st ed., vol. 553. Elsevier Inc., 2015.
http://www.sciencedirect.com/science/article/pii/S0076687914000524

::

    rna_rosetta_cluster.py ade_min.out 20000

Take n * 0.005 (.5%) of all frames and put them into `selected.out`. Then the tool clusters this `selected.out`.
"""
from __future__ import print_function
import platform
import argparse
import os
import subprocess
import math
import logging
import shutil
import time
#limit_clusters = 1  # if you want to change this, then rewrite procedure of stopping!

logging.basicConfig(level=logging.INFO)


def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument('--no_select', action='store_true',
                        help="Don't run selection once again. Use selected.out in the current folder")
    parser.add_argument('--radius-inc-step', type=float, default=0.5,
                        help="radius incremental step, default 0.5")
    parser.add_argument('--limit-clusters', default=5,
                        help="# of clusters")
    parser.add_argument('file', help='ade.out')
    parser.add_argument('n', type=int, help='# of total structures')
    return parser


def get_no_structures_in_first_cluster(fn):
    """Get # of structures in a silent file.

    Args:
         fn (string): a filename to a silent file

    Returns
         int: # of structures in a silent file"""
    p = subprocess.Popen('grep SCORE ' + fn + ' | grep c.0 | wc -l', shell=True,
                         stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.wait()
    stderr = p.stderr.read().strip()
    if stderr:
        print(stderr)
    return int(p.stdout.read().strip())


def get_selected(file, nc):
    """Get selected for clustering"""
    cmd = 'silent_file_sort_and_select.py ' + file + ' -select 1-' + str(nc) + ' -o selected.out'
    logging.info(cmd)
    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    p.wait()
    stderr = p.stderr.read().strip()
    if stderr:
        print(stderr)


def cluster(radius, limit_clusters):
    """Internal function of cluster_loop: It removes cluster.out first."""
    try:
        os.remove("cluster.out")
    except OSError:
        pass

    if platform.system() == "Darwin":
        cmd = 'cluster.macosclangrelease'
    if platform.system() == "Linux":
        cmd = "cluster.default.linuxgccrelease"
    cmd += " -in:file:silent selected.out -in:file:fullatom -out:file:silent_struct_type binary -export_only_low false -out:file:silent cluster.out -cluster:limit_clusters " + str(limit_clusters) + " -cluster:radius " + str(radius)

    #'-out:prefix ade_ "#| tee clustering.out"
    logging.info(cmd)
    os.system(cmd)
    #p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    # p.wait()
    #stderr = p.stderr.read().strip()
    # if stderr:
    #    print stderr


def extract():
    #cmd = "~/src/rna-pdb-tools/rna_tools/utils/rna_rosetta/rna_extract_lowscore_decoys.py cluster0.out 5"
    if platform.system() == "Darwin":
        cmd = 'extract_pdbs.default.macosclangrelease -in::file::silent cluster.out'
    if platform.system() == "Linux":
        cmd = 'extract_pdbs.default.linuxgccrelease -in::file::silent cluster.out'
    os.system(cmd)


def cluster_loop(ns, radius_inc_step, limit_clusters):
    """Go from radius 1 to get 1/6 of structures of ns (# of selected structures)
    in the first cluster, then it stops."""
    radius = 1  # should be 1
    while 1:
        cluster(radius, limit_clusters)
        ns1 = get_no_structures_in_first_cluster('cluster.out')
        print('%i in #1 cluster %i from the total number of structures' % (ns1, ns))
        if ns1 > ns * .16:  # 1/6
            break
        radius += radius_inc_step
    logging.info('radius %i ; %i in #1 cluster %i from the total number of structures' % (radius, ns1, ns))
    return radius, ns1

def get_no_structures(file):
    p = subprocess.Popen('cat ' + file + ' | grep SCORE | wc -l', shell=True,
                         stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stderr = p.stderr.read().strip()
    if stderr:
        print(stderr)
    return int(p.stdout.read().strip()) - 1


def run():
    """Pipline for modeling RNA"""
    args = get_parser().parse_args()
    logging.info(time.strftime("%Y-%m-%d %H:%M"))
    logging.info(args)
    # get_no_structures(args.file) # if you mini then # is the total number of structures
    n = args.n
    nc = int(math.ceil(n * 0.005))  # nc no for clustring
    print('# total:', n, ' # selected:', nc)
    # selection
    if not args.no_select:
        get_selected(args.file, nc)
        ns = get_no_structures('selected.out')
        print('# selected:', ns)

    ns = get_no_structures('selected.out')

    radius, ns1 = cluster_loop(ns, args.radius_inc_step, args.limit_clusters)  # loop over selected to get 1/6 in the biggest cluster
    extract()

    print("# of structures in the biggest cluster", ns1)
    logging.info('radius %f, step %f ; %i in #1 cluster %i from the total number of structures' % (radius, args.radius_inc_step, ns1, ns))

# main
if __name__ == '__main__':
    run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
The output you simply get from the screen, save it it to a file.

Example::

    rna_alignment_get_species.py RF00004.stockholm.stk
    # STOCKHOLM 1.0
    Sorex-araneus-(European-shrew)                     AUCGCU-UCU----CGGCC--UUU-U

Examples 2::

    [dhcp177-lan203] Desktop$ rna_alignment_get_species.py u5_rfam_u5only.stk --verbose
    # STOCKHOLM 1.0
    #=GF WK U5_spliceosomal_RNA
    #=GF NC 39.90
    #=GF RT The spliceosomal snRNAs of Caenorhabditis elegans.
    #=GF TC 40.00
    #=GF RN [3]
    #=GF RM 2339054
    #=GF RL Nucleic Acids Res 1990;18:2633-2642.
    #=GF AU Gardner PP; 0000-0002-7808-1213
    #=GF CC methylation.
    #=GF CB cmcalibrate --mpi CM
    #=GF DR GO; 0046540; U4/U6 x U5 tri-snRNP complex;
    #=GF ID U5
    #=GF SS Published; PMID:2339054; Griffiths-Jones SR
    #=GF RA Thomas J, Lea K, Zucker-Aprison E, Blumenthal T
    #=GF SQ 180
    #=GF SM cmsearch --cpu 4 --verbose --nohmmonly -E 1000 -Z 549862.597050 CM SEQDB
    #=GF DE U5 spliceosomal RNA
    #=GF AC RF00020
    #=GF SE Zwieb C, The uRNA database, PMID:9016512; PMID:18390578
    #=GF GA 40.00
    #=GF BM cmbuild -F CM SEED
    #=GF TP Gene; snRNA; splicing;
    Bos-taurus-(cattle)                                GAUC-GUAUAAAUCUUUCGCCUUUUACUAAAGA-UUUCCG----UGG-A--GA-G
    Sorex-araneus-(European-shrew)                     GAUC-GUAUAAAUCUUUCGCCUUUUACUAAAGA-UUUCCG----UGG-A--GA-G
    Ictidomys-tridecemlineatus-(thirteen-lined-ground- GAUC-GUAUAAAUCUUUCGCCUUUUACUAAAGA-UUUCCG----UGG-A--GA-G
    Monodelphis-domestica-(gray-short-tailed-opossum)  GAUC-GUAUAAAUCUUUCGCCUUUUACUAAAGA-UUUCCG----UGG-A--GA-G
    Oryctolagus-cuniculus-(rabbit)                     GAUC-GUAUAAAUCUUUCGCCUUUUACUAAAGA-UUUCCG----UGG-A--GA-G
    Cavia-porcellus-(domestic-guinea-pig)              GAUC-GUAUAAAUCUUUCGCCUUUUACUAAAGA-UUUCCG----UGG-A--GA-G
    Ochotona-princeps-(American-pika)                  GAUC-GUAUAAAUCUUUCGCCUUUUACUAAAGA-UUUCCG----UGG-A--GA-G

.. note::

  This code has way more code than the name of the script says. This is customized script based on
  some script that did way more.
"""
from __future__ import print_function

from rna_tools.tools.rna_alignment.rna_alignment import RNAalignment
from rna_tools.Seq import RNASequence
import pandas as pd
import argparse
import urllib2

def get_parser():
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter)

    parser.add_argument("-v", "--verbose",
                        action="store_true", help="be verbose")
    parser.add_argument("--debug", action="store_true")
    parser.add_argument('--id-width', type=int, default=50)
    parser.add_argument('--evo-mapping')
    parser.add_argument('--evo-mapping-default', action="store_true")
    parser.add_argument('--one', action="store_true")
    parser.add_argument('--u5', action="store_true")
    parser.add_argument('--calc-energy', action="store_true")
    parser.add_argument('--osfn')
    parser.add_argument("alignment")
    return parser


def ungap(x):
    return x.replace('-', '')


def semi_clean_id(id):
    id = id.split('/')[0]
    return id


def clean_id(id):
    id = id.split('/')[0]
    id = id.split('.')[0]
    return id

def get_species(id, ocfn, verbose=False):
    """
    OS   Leishmania tarentolae
    OC   Eukaryota; Euglenozoa; Kinetoplastida; Trypanosomatidae; Leishmaniinae;
    OC   Leishmania; lizard Leishmania.

    link:
    https://www.ebi.ac.uk/ena/data/view/AANU01000000&display=text&download=txt&filename=AANU01000000.txt
    """
    # clean for AABX02000022.1/363025-363047 -> AABX02000022.1
    id = clean_id(id)

    if ocfn:
        try:
            df = pd.read_csv(ocfn, index_col=0)
        except:
            df = pd.DataFrame()
        if id in df.index:
            os = df[df.index == id]['os'].item()
            return os, ''  #
    # download
    url = "https://www.ebi.ac.uk/ena/data/view/%s&display=text&download=txt&filename=tmp.txt" % id
    response = urllib2.urlopen(url)
    oc = ''
    os = ''
    for l in response:
        if l.startswith('OS'):
            os = l.replace('OS', '')
        if l.startswith('OC'):
            oc += l.replace('OC', '').strip()

    if not os:
        if verbose:
            print(id)
            print(url)
        return None, None

    if ocfn:
        os, oc = os.strip(), oc.strip()
        df.index.name = 'id'
        df = df.append(pd.DataFrame({'os' : [os]}, index=[id])) # 'oc': [oc]
        df.to_csv(ocfn) # , index=False)
    return os, oc


# some default simple mapping
mapping =  [['Metazoa', 'Metazoa'],
           ['Viridiplantae', 'Viridiplantae'],
           ['Saccharomycotina', '(Fungi) Saccharomycotina'],
           ['Euglenozoa', 'Euglenozoa'],
           ['Alveolata', 'Alveolata'],
           ['Amoebozoa', 'Amoebozoa'],
           ['Cryptophyta', 'Cryptophyta'],
           ['Parabasalia', 'Parabasalia'],
           ['Rhizaria', 'Rhizaria'],
           ['Stramenopiles', 'Stramenopiles'],
           ['metagenomes', 'metagenomes'],
           ]


if __name__ == '__main__':
    parser = get_parser()
    args = parser.parse_args()

    a = args.alignment
    name_width = args.id_width

    if args.evo_mapping:
        mapping = eval(open(args.evo_mapping).read().replace('\n', ''))
        if args.verbose:
            print(mapping)

    os_done = []

    cc = 1
    for l in open(a):
        if l.strip():
            if not l.startswith('#') and not l.startswith('//'):
                if args.debug:
                    print(cc)
                    cc += 1
                try:
                    id, seq = l.split()
                except:
                    print('Error in ', l)
                    sys.exit(1)
                energy = ''
                ss = ''
                if False:
                    ################################################################################
                    a = ungap(seq[0:4])
                    b = ungap(seq[-3:])
                    ####################
                    # to the new alignment send only a + b, overwrite the original sequence
                    # print(seq)
                    seq = a + '-' + b
                    # print(seq)

                if args.u5:
                    a = ungap(seq[0:8])
                    b = ungap(seq[-9:])
                    seq = a + '-' + b

                if args.calc_energy:
                    # u5
                    seql = RNASequence(seq.replace('-','').replace('N', 'u'))
                    cst = "((((((((...........))))))))"
                    #AAAUCUUUCGCCUUUUACUAAAGA-UUU
                    #((((((((...........))))).)))
                    energy, ss = seql.predict_ss(method="mcfold", constraints=cst, verbose=args.verbose)
                    # u6
                    #pass
                    ## # u6atac
                    ## a = 'g' + ungap(seq[0:5])
                    ## b = ungap(seq[-4:]) + 'c'
                    ## loop = "guaa"
                    ## seql = RNASequence(a + loop + b)
                    ## cst = '((((.((..))))))'
                    ## if args.verbose:
                    ##     print('seq %s' % seq)
                    ##     print(a + loop + b)
                    ##     print(cst)
                    ## energy, ss = seql.predict_ss(method="mcfold", constraints=cst, verbose=args.verbose)
                    ## if args.verbose: print(energy, ss)
                ################################################################################
                os, oc = get_species(id, args.osfn)
                if not os:
                    os = id
                os = os.replace('.', '_') # remove dots from here
                # check if os if it's there already
                c = 1
                while 1:
                    if os not in os_done:
                        os_done.append(os) # Tupaia-chinensis-(Chinese-tree-shrew).1.2.3.4.5.6. fuck!
                        break
                    if len(os.split('.')) == 2:
                        os = os.replace('.' + str(c - 1), '.' + str(c))
                    else:
                        os += '.' + str(c)
                    if args.debug:
                        pass
                    c += 1
                ################################################################################
                if args.evo_mapping or args.evo_mapping_default:
                    for m in mapping: # m is ['Parabasalia', 'Parabasalia']
                        if oc:
                            if m[0] in oc:
                                group = m[1]
                        else:
                            group = '???'
                    #
                    #print((os.replace(' ', '-')[:name_width] + '[' + group.replace(' ','-') + ']' + semi_clean_id(id) + '|' + str(round(energy,2)) +
                    #       '|' + a + '-' + b + ss).ljust(name_width), seq.strip()) # clean_id(id)
                    if energy:
                        print((os.replace(' ', '-')[:name_width] + '[' + group.replace(' ','-') + ']' + semi_clean_id(id) + '|' + str(round(energy,2)) +
                               '|' + ss).ljust(name_width), seq.strip()) # clean_id(id)
                    else:
                        print((group.replace(' ','-') + '|' + os.replace(' ', '-')[:name_width] + '|' + semi_clean_id(id) + ss).ljust(name_width), seq.strip())

                else:
                    # this is no group
                    # ech, this is super messy, sorry!
                    group = ''
                    print((os.strip().replace(' ', '-')[:name_width] + str(energy)).ljust(name_width), seq.strip())

                if args.one:
                    break
                #else:
                #    print(os.replace(' ', '-') + str(energy)[:name_width].ljust(name_width), seq.strip())

            elif '#=GC SS_cons' in l:
                ss = l.replace('#=GC SS_cons', '')
                print('#=GC SS_cons'.ljust(name_width), ss.strip())

            # OK, i'm not sure, if this should be RF_cons or RF
            elif '#=GC RF_cons' in l:
                ss = l.replace('#=GC SS_cons', '')
                print('#=GC RF_cons'.ljust(name_width), ss.strip())
            elif '#=GC RF' in l:
                ss = l.replace('#=GC RF', '')
                print('#=GC RF'.ljust(name_width), ss.strip())
            else:
                print(l.strip())
